#!/usr/bin/python
#
# Copyright (C) 2006 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


__author__ = 'api.jscudder@gmail.com (Jeff Scudder)'


import unittest
try:
  from xml.etree import ElementTree
except ImportError:
  from elementtree import ElementTree
import atom 
import gdata
from gdata import test_data
import gdata.calendar


class CalendarFeedTest(unittest.TestCase):
  
  def setUp(self):
    self.calendar_feed = gdata.calendar.CalendarListFeedFromString(
        test_data.CALENDAR_FEED)

  def testEntryCount(self):
    # Assert the number of items in the feed of calendars
    self.assertEquals(len(self.calendar_feed.entry),2)
    
      

  def testToAndFromString(self):
    # Assert the appropriate type for each entry
    for an_entry in self.calendar_feed.entry:
      self.assert_(isinstance(an_entry, gdata.calendar.CalendarListEntry), 
          'Entry must be an instance of CalendarListEntry')

    # Regenerate feed from xml text
    new_calendar_feed = ( 
        gdata.calendar.CalendarListFeedFromString(str(self.calendar_feed)))
    for an_entry in new_calendar_feed.entry:
      self.assert_(isinstance(an_entry, gdata.calendar.CalendarListEntry), 
          'Entry in regenerated feed must be an instance of CalendarListEntry')

  def testAuthor(self):
    """Tests the existence of a <atom:author> and verifies the name and email"""

    # Assert that each element in the feed author list is an atom.Author
    for an_author in self.calendar_feed.author:
      self.assert_(isinstance(an_author, atom.Author), 
          "Calendar feed <atom:author> element must be an instance of " +
          "atom.Author: %s" % an_author)

    # Assert the feed author name is as expected
    self.assertEquals(self.calendar_feed.author[0].name.text, 'GData Ops Demo')

    # Assert the feed author name is as expected
    self.assertEquals(self.calendar_feed.author[0].email.text, 
        'gdata.ops.demo@gmail.com')

    # Assert one of the values for an entry author
    self.assertEquals(self.calendar_feed.entry[0].author[0].name.text, 
        'GData Ops Demo')
    self.assertEquals(self.calendar_feed.entry[0].author[0].email.text, 
        'gdata.ops.demo@gmail.com')

  def testId(self):
    """Tests the existence of a <atom:id> in the feed and entries 
    and verifies the value"""

    # Assert the feed id exists and is an atom.Id
    self.assert_(isinstance(self.calendar_feed.id, atom.Id), 
        "Calendar feed <atom:id> element must be an instance of atom.Id: %s" % (
        self.calendar_feed.id)) 

    # Assert the feed id value is as expected 
    self.assertEquals(self.calendar_feed.id.text, 
        'http://www.google.com/calendar/feeds/default')

    # Assert that each entry has an id which is an atom.Id
    for an_entry in self.calendar_feed.entry:
      self.assert_(isinstance(an_entry.id, atom.Id),
          "Calendar entry <atom:id> element must be an instance of " +
          "atom.Id: %s" % an_entry.id)

    # Assert one of the values for an id
    self.assertEquals(self.calendar_feed.entry[1].id.text, 
        'http://www.google.com/calendar/feeds/default/' +
        'jnh21ovnjgfph21h32gvms2758%40group.calendar.google.com')

  def testPublished(self):
    """Tests the existence of a <atom:published> in the entries 
    and verifies the value"""

    # Assert that each entry has a published value which is an atom.Published
    for an_entry in self.calendar_feed.entry:
      self.assert_(isinstance(an_entry.published, atom.Published),
          "Calendar entry <atom:published> element must be an instance of " +
          "atom.Published: %s" % an_entry.published)

    # Assert one of the values for published is as expected
    self.assertEquals(self.calendar_feed.entry[1].published.text, 
       '2007-03-20T22:48:57.837Z')

  def testUpdated(self):
    """Tests the existence of a <atom:updated> in the feed and the entries 
    and verifies the value"""

    # Assert that the feed updated element exists and is an atom.Updated
    self.assert_(isinstance(self.calendar_feed.updated, atom.Updated), 
        "Calendar feed <atom:updated> element must be an instance of " +
        "atom.Updated: %s" % self.calendar_feed.updated)

    # Assert that each entry has a updated value which is an atom.Updated
    for an_entry in self.calendar_feed.entry:
      self.assert_(isinstance(an_entry.updated, atom.Updated),
          "Calendar entry <atom:updated> element must be an instance of" + 
          "atom.Updated: %s" % an_entry.updated)

    # Assert the feed updated value is as expected
    self.assertEquals(self.calendar_feed.updated.text, 
        '2007-03-20T22:48:57.833Z')

    # Assert one of the values for updated
    self.assertEquals(self.calendar_feed.entry[0].updated.text, 
        '2007-03-20T22:48:52.000Z')

  def testTitle(self):
    """Tests the existence of a <atom:title> in the feed and the entries and 
    verifies the value"""

    # Assert that the feed title element exists and is an atom.Title
    self.assert_(isinstance(self.calendar_feed.title, atom.Title), 
        "Calendar feed <atom:title> element must be an instance of " + 
        "atom.Title: %s" % self.calendar_feed.title)

    # Assert that each entry has a title value which is an atom.Title
    for an_entry in self.calendar_feed.entry:
      self.assert_(isinstance(an_entry.title, atom.Title),
          "Calendar entry <atom:title> element must be an instance of " +
          "atom.Title: %s" % an_entry.title)

    # Assert the feed title value is as expected
    self.assertEquals(self.calendar_feed.title.text, 
        'GData Ops Demo\'s Calendar List')

    # Assert one of the values for title 
    self.assertEquals(self.calendar_feed.entry[0].title.text, 'GData Ops Demo')

  def testColor(self):
    """Tests the existence of a <gCal:color> and verifies the value"""
    
    # Assert the color is present and is a gdata.calendar.Color
    for an_entry in self.calendar_feed.entry:
      self.assert_(isinstance(an_entry.color, gdata.calendar.Color), 
          "Calendar feed <gCal:color> element must be an instance of " +
          "gdata.calendar.Color: %s" % an_entry.color)

    # Assert the color value is as expected
    self.assertEquals(self.calendar_feed.entry[0].color.value, '#2952A3')

  def testAccessLevel(self):
    """Tests the existence of a <gCal:accesslevel> element and verifies the
    value"""

    # Assert the access_level is present and is a gdata.calendar.AccessLevel
    for an_entry in self.calendar_feed.entry:
      self.assert_(isinstance(an_entry.access_level, gdata.calendar.AccessLevel), 
          "Calendar feed <gCal:accesslevel> element must be an instance of " +
          "gdata.calendar.AccessLevel: %s" % an_entry.access_level)

    # Assert the access_level value is as expected
    self.assertEquals(self.calendar_feed.entry[0].access_level.value, 'owner')

  def testTimezone(self):
    """Tests the existence of a <gCal:timezone> element and verifies the
     value"""

    # Assert the timezone is present and is a gdata.calendar.Timezone
    for an_entry in self.calendar_feed.entry:
      self.assert_(isinstance(an_entry.timezone, gdata.calendar.Timezone), 
          "Calendar feed <gCal:timezone> element must be an instance of " +
          "gdata.calendar.Timezone: %s" % an_entry.timezone)

    # Assert the timezone value is as expected
    self.assertEquals(self.calendar_feed.entry[0].timezone.value, 
        'America/Los_Angeles')

  def testHidden(self):
    """Tests the existence of a <gCal:hidden> element and verifies the
     value"""

    # Assert the hidden is present and is a gdata.calendar.Hidden
    for an_entry in self.calendar_feed.entry:
      self.assert_(isinstance(an_entry.hidden, gdata.calendar.Hidden), 
          "Calendar feed <gCal:hidden> element must be an instance of " +
          "gdata.calendar.Hidden: %s" % an_entry.hidden)

    # Assert the hidden value is as expected
    self.assertEquals(self.calendar_feed.entry[0].hidden.value, 'false')

  def testOpenSearch(self):
    """Tests the existence of <openSearch:startIndex>"""
    # Assert that the elements exist and are the appropriate type
    self.assert_(isinstance(self.calendar_feed.start_index, gdata.StartIndex),
          "Calendar feed <openSearch:startIndex> element must be an " +
          "instance of gdata.StartIndex: %s" % self.calendar_feed.start_index)

    # Assert the values for each openSearch element are as expected
    self.assertEquals(self.calendar_feed.start_index.text, '1')

  def testGenerator(self):
    """Tests the existence of <atom:generator> and verifies the value"""

    # Assert that the element exists and is of the appropriate type
    self.assert_(isinstance(self.calendar_feed.generator, atom.Generator),
          "Calendar feed <atom:generator> element must be an instance of " +
          "atom.Generator: %s" % self.calendar_feed.generator)

    # Assert the generator version, uri and text are as expected
    self.assertEquals(self.calendar_feed.generator.text, 'Google Calendar')
    self.assertEquals(self.calendar_feed.generator.version, '1.0')
    self.assertEquals(self.calendar_feed.generator.uri, 
        'http://www.google.com/calendar')

  def testEntryLink(self):
    """Makes sure entry links in the private composite feed are parsed."""

    entry = gdata.calendar.CalendarEventEntryFromString(
        test_data.RECURRENCE_EXCEPTION_ENTRY)

    self.assert_(isinstance(entry.recurrence_exception, list))
    self.assert_(isinstance(entry.recurrence_exception[0].entry_link, 
        gdata.EntryLink))
    self.assert_(isinstance(entry.recurrence_exception[0].entry_link.entry,
        gdata.calendar.CalendarEventEntry))
    self.assertEquals(
        entry.recurrence_exception[0].entry_link.entry.author[0].name.text, 
        'gdata ops')

  def testSequence(self):
    entry = gdata.calendar.CalendarEventEntry(
        sequence=gdata.calendar.Sequence(value='1'))
    entry2 = gdata.calendar.CalendarEventEntryFromString(str(entry))
    self.assertEqual(entry.sequence.value, entry2.sequence.value)
    entry = gdata.calendar.CalendarEventEntryFromString(
        '<entry xmlns="%s"><sequence xmlns="%s" value="7" /></entry>' % (
            atom.ATOM_NAMESPACE, gdata.calendar.GCAL_NAMESPACE))
    self.assertEqual(entry.sequence.value, '7')

  def testOriginalEntry(self):
    """Make sure original entry in the private composite feed are parsed."""

    entry = gdata.calendar.CalendarEventEntryFromString(
        test_data.RECURRENCE_EXCEPTION_ENTRY)

    self.assertEquals(
        entry.recurrence_exception[0].entry_link.entry.original_event.id,
        'i7lgfj69mjqjgnodklif3vbm7g')


class CalendarFeedTestRegenerated(CalendarFeedTest):
  def setUp(self):
    old_calendar_feed = (
        gdata.calendar.CalendarListFeedFromString(test_data.CALENDAR_FEED))
    self.calendar_feed = (
        gdata.calendar.CalendarListFeedFromString(str(old_calendar_feed)))
    tree = ElementTree.fromstring(str(old_calendar_feed))


class CalendarEventFeedTest(unittest.TestCase):
  
  def setUp(self):
    self.calendar_event_feed = (
        gdata.calendar.CalendarEventFeedFromString(
            test_data.CALENDAR_FULL_EVENT_FEED))

  def testEntryCount(self):
    # Assert the number of items in the feed of events
    self.assertEquals(len(self.calendar_event_feed.entry),11)

  def testToAndFromString(self):
    # Assert the appropriate type for each entry
    for an_entry in self.calendar_event_feed.entry:
      self.assert_(isinstance(an_entry, gdata.calendar.CalendarEventEntry), 
          "Entry must be an instance of a CalendarEventEntry")

    # Regenerate feed from xml text
    new_calendar_event_feed = gdata.calendar.CalendarEventFeedFromString(
        str(self.calendar_event_feed))
    for an_entry in new_calendar_event_feed.entry:
      self.assert_(isinstance(an_entry, gdata.calendar.CalendarEventEntry), 
          "Entry in regenerated feed must be an instance of CalendarEventEntry")

  def testAuthor(self):
    """Tests the existence of a <atom:author> and verifies the name and email"""

    # Assert that each element in the feed author list is an atom.Author
    for an_author in self.calendar_event_feed.author:
      self.assert_(isinstance(an_author, atom.Author), 
          "Calendar event feed <atom:author> element must be an instance of " +
          "atom.Author: %s" % an_author)

    # Assert the feed author name is as expected
    self.assertEquals(self.calendar_event_feed.author[0].name.text, 
        'GData Ops Demo')

    # Assert the feed author name is as expected
    self.assertEquals(self.calendar_event_feed.author[0].email.text, 
        'gdata.ops.demo@gmail.com')

    # Assert one of the values for an entry author
    self.assertEquals(self.calendar_event_feed.entry[0].author[0].name.text, 
        'GData Ops Demo')
    self.assertEquals(self.calendar_event_feed.entry[0].author[0].email.text, 
        'gdata.ops.demo@gmail.com')

  def testId(self):
    """Tests the existence of a <atom:id> in the feed and entries and 
    verifies the value"""

    # Assert the feed id exists and is an atom.Id
    self.assert_(isinstance(self.calendar_event_feed.id, atom.Id), 
        "Calendar event feed <atom:id> element must be an instance of " +
        "atom.Id: %s" % self.calendar_event_feed.id)

    # Assert the feed id value is as expected 
    self.assertEquals(self.calendar_event_feed.id.text, 
        'http://www.google.com/calendar/feeds/default/private/full')

    # Assert that each entry has an id which is an atom.Id
    for an_entry in self.calendar_event_feed.entry:
      self.assert_(isinstance(an_entry.id, atom.Id),
          "Calendar event entry <atom:id> element must be an " +
          "instance of atom.Id: %s" % an_entry.id)

    # Assert one of the values for an id
    self.assertEquals(self.calendar_event_feed.entry[1].id.text, 
        'http://www.google.com/calendar/feeds/default/private/full/' + 
        '2qt3ao5hbaq7m9igr5ak9esjo0')

  def testPublished(self):
    """Tests the existence of a <atom:published> in the entries and 
    verifies the value"""

    # Assert that each entry has a published value which is an atom.Published
    for an_entry in self.calendar_event_feed.entry:
      self.assert_(isinstance(an_entry.published, atom.Published),
          "Calendar event entry <atom:published> element must be an instance " +
          "of atom.Published: %s" % an_entry.published)

    # Assert one of the values for published is as expected
    self.assertEquals(self.calendar_event_feed.entry[1].published.text, 
        '2007-03-20T21:26:04.000Z')

  def testUpdated(self):
    """Tests the existence of a <atom:updated> in the feed and the entries and 
    verifies the value"""

    # Assert that the feed updated element exists and is an atom.Updated
    self.assert_(isinstance(self.calendar_event_feed.updated, atom.Updated), 
        "Calendar feed <atom:updated> element must be an instance of " +
        "atom.Updated: %s" % self.calendar_event_feed.updated)

    # Assert that each entry has a updated value which is an atom.Updated
    for an_entry in self.calendar_event_feed.entry:
      self.assert_(isinstance(an_entry.updated, atom.Updated),
          "Calendar event entry <atom:updated> element must be an instance " +
          "of atom.Updated: %s" % an_entry.updated)

    # Assert the feed updated value is as expected
    self.assertEquals(self.calendar_event_feed.updated.text, 
        '2007-03-20T21:29:57.000Z')

    # Assert one of the values for updated
    self.assertEquals(self.calendar_event_feed.entry[3].updated.text, 
        '2007-03-20T21:25:46.000Z')

  def testTitle(self):
    """Tests the existence of a <atom:title> in the feed and the entries 
    and verifies the value"""

    # Assert that the feed title element exists and is an atom.Title
    self.assert_(isinstance(self.calendar_event_feed.title, atom.Title), 
        "Calendar feed <atom:title> element must be an instance of " +
        "atom.Title: %s" % self.calendar_event_feed.title)

    # Assert that each entry has a title value which is an atom.Title
    for an_entry in self.calendar_event_feed.entry:
      self.assert_(isinstance(an_entry.title, atom.Title),
          "Calendar event entry <atom:title> element must be an instance of " +
          "atom.Title: %s" % an_entry.title)

    # Assert the feed title value is as expected
    self.assertEquals(self.calendar_event_feed.title.text, 'GData Ops Demo')

    # Assert one of the values for title 
    self.assertEquals(self.calendar_event_feed.entry[0].title.text, 
        'test deleted')

  def testPostLink(self):
    """Tests the existence of a <atom:link> with a rel='...#post' 
    and verifies the value"""

    # Assert that each link in the feed is an atom.Link
    for a_link in self.calendar_event_feed.link:
      self.assert_(isinstance(a_link, atom.Link),
          "Calendar event entry <atom:link> element must be an instance of " +
          "atom.Link: %s" % a_link)

    # Assert post link exists
    self.assert_(self.calendar_event_feed.GetPostLink() is not None)

    # Assert the post link value is as expected
    self.assertEquals(self.calendar_event_feed.GetPostLink().href, 
        'http://www.google.com/calendar/feeds/default/private/full')

  def testEditLink(self):
    """Tests the existence of a <atom:link> with a rel='edit' in each entry 
    and verifies the value"""

    # Assert that each link in the feed is an atom.Link
    for a_link in self.calendar_event_feed.link:
      self.assert_(isinstance(a_link, atom.Link),
          "Calendar event entry <atom:link> element must be an instance of " + 
          "atom.Link: %s" % a_link)

    # Assert edit link exists
    for a_entry in self.calendar_event_feed.entry:
      self.assert_(a_entry.GetEditLink() is not None)

    # Assert the edit link value is as expected
    self.assertEquals(self.calendar_event_feed.entry[0].GetEditLink().href, 
        'http://www.google.com/calendar/feeds/default/private/full/o99flmgm' +
        'kfkfrr8u745ghr3100/63310109397')
    self.assertEquals(self.calendar_event_feed.entry[0].GetEditLink().type, 
        'application/atom+xml')

  def testOpenSearch(self):
    """Tests the existence of <openSearch:totalResults>, 
    <openSearch:startIndex>, <openSearch:itemsPerPage>"""

    # Assert that the elements exist and are the appropriate type
    self.assert_(isinstance(self.calendar_event_feed.total_results, 
        gdata.TotalResults),
        "Calendar event feed <openSearch:totalResults> element must be an " +
        "instance of gdata.TotalResults: %s" % (
        self.calendar_event_feed.total_results))
    self.assert_(
        isinstance(self.calendar_event_feed.start_index, gdata.StartIndex),
        "Calendar event feed <openSearch:startIndex> element must be an " +
        "instance of gdata.StartIndex: %s" % (
        self.calendar_event_feed.start_index))
    self.assert_(
        isinstance(self.calendar_event_feed.items_per_page, gdata.ItemsPerPage),
        "Calendar event feed <openSearch:itemsPerPage> element must be an " +
        "instance of gdata.ItemsPerPage: %s" % (
        self.calendar_event_feed.items_per_page))

    # Assert the values for each openSearch element are as expected
    self.assertEquals(self.calendar_event_feed.total_results.text, '10')
    self.assertEquals(self.calendar_event_feed.start_index.text, '1')
    self.assertEquals(self.calendar_event_feed.items_per_page.text, '25')

  def testGenerator(self):
    """Tests the existence of <atom:generator> and verifies the value"""

    # Assert that the element exists and is of the appropriate type
    self.assert_(isinstance(self.calendar_event_feed.generator, atom.Generator),
          "Calendar event feed <atom:generator> element must be an instance " +
          "of atom.Generator: %s" % self.calendar_event_feed.generator)

    # Assert the generator version, uri and text are as expected
    self.assertEquals(self.calendar_event_feed.generator.text, 
        'Google Calendar')
    self.assertEquals(self.calendar_event_feed.generator.version, '1.0')
    self.assertEquals(self.calendar_event_feed.generator.uri, 
        'http://www.google.com/calendar')

  def testCategory(self):
    """Tests the existence of <atom:category> and verifies the value"""

    # Assert that the element exists and is of the appropriate type and value
    for a_category in self.calendar_event_feed.category:
      self.assert_(isinstance(a_category, atom.Category),
          "Calendar event feed <atom:category> element must be an instance " +
          "of atom.Category: %s" % a_category)
      self.assertEquals(a_category.scheme, 
          'http://schemas.google.com/g/2005#kind')
      self.assertEquals(a_category.term, 
          'http://schemas.google.com/g/2005#event')

    for an_event in self.calendar_event_feed.entry:
      for a_category in an_event.category:
        self.assert_(isinstance(a_category, atom.Category),
            "Calendar event feed entry <atom:category> element must be an " +
            "instance of atom.Category: %s" % a_category)
        self.assertEquals(a_category.scheme, 
            'http://schemas.google.com/g/2005#kind')
        self.assertEquals(a_category.term, 
            'http://schemas.google.com/g/2005#event')


  def testSendEventNotifications(self):
    """Test the existence of <gCal:sendEventNotifications> 
    and verifies the value"""

    # Assert that the element exists and is of the appropriate type and value
    for an_event in self.calendar_event_feed.entry:
      self.assert_(isinstance(an_event.send_event_notifications, 
          gdata.calendar.SendEventNotifications),
          ("Calendar event feed entry <gCal:sendEventNotifications> element " +
          "must be an instance of gdata.calendar.SendEventNotifications: %s") % (
          an_event.send_event_notifications,))
   
    # Assert the <gCal:sendEventNotifications> are as expected 
    self.assertEquals(
        self.calendar_event_feed.entry[0].send_event_notifications.value,
        'false')

    self.assertEquals(
        self.calendar_event_feed.entry[2].send_event_notifications.value,
        'true')

  def testQuickAdd(self):
    """Test the existence of <gCal:quickadd> 
    and verifies the value"""

    entry = gdata.calendar.CalendarEventEntry()
    entry.quick_add = gdata.calendar.QuickAdd(value='true')
    unmarshalled_entry = entry.ToString()
    tag = '{%s}quickadd' % (gdata.calendar.GCAL_NAMESPACE)
    marshalled_entry = ElementTree.fromstring(unmarshalled_entry).find(tag)
    self.assert_(marshalled_entry.attrib['value'],'true')
    self.assert_(marshalled_entry.tag,tag)

  def testEventStatus(self):
    """Test the existence of <gd:eventStatus> 
    and verifies the value"""

    # Assert that the element exists and is of the appropriate type and value
    for an_event in self.calendar_event_feed.entry:
      self.assert_(isinstance(an_event.event_status, 
          gdata.calendar.EventStatus),
          ("Calendar event feed entry <gd:eventStatus> element " +
          "must be an instance of gdata.calendar.EventStatus: %s") % (
          an_event.event_status,))
   
    # Assert the <gd:eventStatus> are as expected 
    self.assertEquals(
        self.calendar_event_feed.entry[0].event_status.value,
        'CANCELED')

    self.assertEquals(
        self.calendar_event_feed.entry[1].event_status.value,
        'CONFIRMED')

  def testComments(self):
    """Tests the existence of <atom:comments> and verifies the value"""

    # Assert that the element exists and is of the appropriate type and value
    for an_event in self.calendar_event_feed.entry:
      self.assert_(an_event.comments is None or isinstance(an_event.comments, 
          gdata.calendar.Comments),
          ("Calendar event feed entry <gd:comments> element " +
          "must be an instance of gdata.calendar.Comments: %s") % (
          an_event.comments,))

  def testVisibility(self):
    """Test the existence of <gd:visibility> and verifies the value"""

    # Assert that the element exists and is of the appropriate type and value
    for an_event in self.calendar_event_feed.entry:
      self.assert_(isinstance(an_event.visibility, 
          gdata.calendar.Visibility),
          ("Calendar event feed entry <gd:visibility> element " +
          "must be an instance of gdata.calendar.Visibility: %s") % (
          an_event.visibility,))
   
    # Assert the <gd:visibility> are as expected 
    self.assertEquals(
        self.calendar_event_feed.entry[0].visibility.value,
        'DEFAULT')

    self.assertEquals(
        self.calendar_event_feed.entry[1].visibility.value,
        'PRIVATE')

    self.assertEquals(
        self.calendar_event_feed.entry[2].visibility.value,
        'PUBLIC')

  def testTransparency(self):
    """Test the existence of <gd:transparency> and verifies the value"""

    # Assert that the element exists and is of the appropriate type and value
    for an_event in self.calendar_event_feed.entry:
      self.assert_(isinstance(an_event.transparency, 
          gdata.calendar.Transparency),
          ("Calendar event feed entry <gd:transparency> element " +
          "must be an instance of gdata.calendar.Transparency: %s") % (
          an_event.transparency,))
   
    # Assert the <gd:transparency> are as expected 
    self.assertEquals(
        self.calendar_event_feed.entry[0].transparency.value,
        'OPAQUE')

    self.assertEquals(
        self.calendar_event_feed.entry[1].transparency.value,
        'OPAQUE')

    self.assertEquals(
        self.calendar_event_feed.entry[2].transparency.value,
        'OPAQUE')
 
    # TODO: TEST VALUES OF VISIBILITY OTHER THAN OPAQUE

  def testWhere(self):
    """Tests the existence of a <gd:where> in the entries 
    and verifies the value"""

    # Assert that each entry has a where value which is an gdata.calendar.Where
    for an_entry in self.calendar_event_feed.entry:
      for a_where in an_entry.where:
        self.assert_(isinstance(a_where, gdata.calendar.Where),
            "Calendar event entry <gd:where> element must be an instance of " +
            "gdata.calendar.Where: %s" % a_where)

    # Assert one of the values for where is as expected
    self.assertEquals(self.calendar_event_feed.entry[1].where[0].value_string, 
        'Dolores Park with Kim')

  def testWhenAndReminder(self):
    """Tests the existence of a <gd:when> and <gd:reminder> in the entries 
    and verifies the values"""

    # Assert that each entry's when value is a gdata.calendar.When
    # Assert that each reminder is a gdata.calendar.Reminder 
    for an_entry in self.calendar_event_feed.entry:
      for a_when in an_entry.when:
        self.assert_(isinstance(a_when, gdata.calendar.When),
            "Calendar event entry <gd:when> element must be an instance " +
            "of gdata.calendar.When: %s" % a_when)
        for a_reminder in a_when.reminder:
          self.assert_(isinstance(a_reminder, gdata.calendar.Reminder),
              "Calendar event entry <gd:reminder> element must be an " +
              "instance of gdata.calendar.Reminder: %s" % a_reminder)

    # Assert one of the values for when is as expected
    self.assertEquals(self.calendar_event_feed.entry[0].when[0].start_time, 
        '2007-03-23T12:00:00.000-07:00')
    self.assertEquals(self.calendar_event_feed.entry[0].when[0].end_time, 
        '2007-03-23T13:00:00.000-07:00')

    # Assert the reminder child of when is as expected
    self.assertEquals(
        self.calendar_event_feed.entry[0].when[0].reminder[0].minutes, '10')
    self.assertEquals(
        self.calendar_event_feed.entry[1].when[0].reminder[0].minutes, '20')

  def testBatchRequestParsing(self):
    batch_request = gdata.calendar.CalendarEventFeedFromString(
        test_data.CALENDAR_BATCH_REQUEST)
    self.assertEquals(len(batch_request.entry), 4)
    # Iterate over the batch request entries and match the operation with 
    # the batch id. These values are hard coded to match the test data.
    for entry in batch_request.entry:
      if entry.batch_id.text == '1':
        self.assertEquals(entry.batch_operation.type, 'insert')
      if entry.batch_id.text == '2':
        self.assertEquals(entry.batch_operation.type, 'query')
      if entry.batch_id.text == '3':
        self.assertEquals(entry.batch_operation.type, 'update')
        self.assertEquals(entry.title.text, 'Event updated via batch')
      if entry.batch_id.text == '4':
        self.assertEquals(entry.batch_operation.type, 'delete')
        self.assertEquals(entry.id.text, 
                          'http://www.google.com/calendar/feeds/default/'
                          'private/full/d8qbg9egk1n6lhsgq1sjbqffqc')
        self.assertEquals(entry.GetEditLink().href, 
                          'http://www.google.com/calendar/feeds/default/'
                          'private/full/d8qbg9egk1n6lhsgq1sjbqffqc/'
                          '63326018324')

  def testBatchResponseParsing(self):
    batch_response = gdata.calendar.CalendarEventFeedFromString(
        test_data.CALENDAR_BATCH_RESPONSE)
    self.assertEquals(len(batch_response.entry), 4)
    for entry in batch_response.entry:
      if entry.batch_id.text == '1':
        self.assertEquals(entry.batch_operation.type, 'insert')
        self.assertEquals(entry.batch_status.code, '201')
        self.assertEquals(entry.batch_status.reason, 'Created')
        self.assertEquals(entry.id.text, 'http://www.google.com/calendar/'
                          'feeds/default/private/full/'
                          'n9ug78gd9tv53ppn4hdjvk68ek')
      if entry.batch_id.text == '2':
        self.assertEquals(entry.batch_operation.type, 'query')
      if entry.batch_id.text == '3':
        self.assertEquals(entry.batch_operation.type, 'update')
      if entry.batch_id.text == '4':
        self.assertEquals(entry.batch_operation.type, 'delete')
        self.assertEquals(entry.id.text, 'http://www.google.com/calendar/'
                          'feeds/default/private/full/'
                          'd8qbg9egk1n6lhsgq1sjbqffqc')

  # TODO add reminder tests for absolute_time and hours/seconds (if possible)
  # TODO test recurrence and recurrenceexception
  # TODO test originalEvent 

class CalendarWebContentTest(unittest.TestCase):
  def setUp(self):
    self.calendar_event_feed = (
        gdata.calendar.CalendarEventFeedFromString(
            test_data.CALENDAR_FULL_EVENT_FEED))
    
  def testAddSimpleWebContentEventEntry(self):
    """Verifies that we can add a web content link to an event entry."""
    
    title = "Al Einstein's Birthday!"
    href = 'http://gdata.ops.demo.googlepages.com/birthdayicon.gif'
    type = 'image/jpeg'
    url = 'http://gdata.ops.demo.googlepages.com/einstein.jpg'
    width = '300'
    height = '225'
    
    # Create a web content event
    event = gdata.calendar.CalendarEventEntry()    
    web_content = gdata.calendar.WebContent(url=url, width=width, height=height)
    web_content_link = gdata.calendar.WebContentLink(title=title, 
        href=href, link_type=type, web_content=web_content)
    event.link.append(web_content_link)
    
    # Verify the web content link exists and contains the expected data    
    web_content_link = event.GetWebContentLink()
    self.assertValidWebContentLink(title, href, type, web_content_link)
    
    # Verify the web content element exists and contains the expected data    
    web_content_element = web_content_link.web_content
    self.assertValidSimpleWebContent(url, width, height, web_content_element)
 
  def testAddWebContentGadgetEventEntry(self):
    """Verifies that we can add a web content gadget link to an event entry."""

    title = "Date and Time Gadget"
    href = 'http://gdata.ops.demo.googlepages.com/birthdayicon.gif'
    url = 'http://google.com/ig/modules/datetime.xml'
    type = 'application/x-google-gadgets+xml'
    width = '300'
    height = '200'
    pref_name = 'color'
    pref_value = 'green'

    # Create a web content event
    event = gdata.calendar.CalendarEventEntry()
    web_content = gdata.calendar.WebContent(url=url, width=width, height=height)
    web_content.gadget_pref.append(
        gdata.calendar.WebContentGadgetPref(name=pref_name, value=pref_value))
    web_content_link = gdata.calendar.WebContentLink(title=title,
        href=href, web_content=web_content, link_type=type)
    event.link.append(web_content_link)

    # Verify the web content link exists and contains the expected data    
    web_content_link = event.GetWebContentLink()
    self.assertValidWebContentLink(title, href, type, web_content_link)

    # Verify the web content element exists and contains the expected data    
    web_content_element = web_content_link.web_content
    self.assertValidWebContentGadget(url, width, height, 
        pref_name, pref_value, web_content_element)
 
  def testFromXmlToSimpleWebContent(self):
    """Verifies that we can read a web content link from an event entry."""

    # Expected values (from test_data.py file)
    title = 'World Cup'
    href = 'http://www.google.com/calendar/images/google-holiday.gif'
    type = 'image/gif'
    url = 'http://www.google.com/logos/worldcup06.gif'
    width = '276'
    height = '120'

    # Note: The tenth event entry contains web content
    web_content_event = self.calendar_event_feed.entry[9]

    # Verify the web content link exists and contains the expected data
    web_content_link = web_content_event.GetWebContentLink()
    self.assertValidWebContentLink(title, href, type, web_content_link)
    
    # Verify the web content element exists and contains the expected data
    web_content_element = web_content_link.web_content
    self.assertValidSimpleWebContent(url, width, height, web_content_element)

  def testFromXmlToWebContentGadget(self):
    """Verifies that we can read a web content link from an event entry."""

    # Expected values (from test_data.py file)
    title = 'Date and Time Gadget'
    href = 'http://gdata.ops.demo.googlepages.com/birthdayicon.gif'
    url = 'http://google.com/ig/modules/datetime.xml'
    type = 'application/x-google-gadgets+xml'
    width = '300'
    height = '136'
    pref_name = 'color'
    pref_value = 'green'

    # Note: The eleventh event entry contains web content
    web_content_event = self.calendar_event_feed.entry[10]
  
    # Verify the web content link exists and contains the expected data
    web_content_link = web_content_event.GetWebContentLink()
    self.assertValidWebContentLink(title, href, type, web_content_link)

    # Verify the web content element exists and contains the expected data
    web_content_element = web_content_link.web_content
    self.assertValidWebContentGadget(url, width, height, pref_name, 
        pref_value, web_content_element)
    
  def assertValidWebContentLink(self, expected_title=None, expected_href=None, 
      expected_type=None, web_content_link=None):
    """Asserts that the web content link is the correct type and contains the
    expected values"""
    
    self.assert_(isinstance(web_content_link, gdata.calendar.WebContentLink),
          "Web content link element must be an " +
          "instance of gdata.calendar.WebContentLink: %s" % web_content_link)
    expected_rel = '%s/%s' % (gdata.calendar.GCAL_NAMESPACE, 'webContent')
    self.assertEquals(expected_rel, web_content_link.rel)
    self.assertEqual(expected_title, web_content_link.title)
    self.assertEqual(expected_href, web_content_link.href)
    self.assertEqual(expected_type, web_content_link.type)

  def assertValidSimpleWebContent(self, expected_url=None, expected_width=None, 
      expected_height=None, web_content_element=None):
    """Asserts that the web content element is the correct type and contains
    the expected values"""

    self.assert_(isinstance(web_content_element, gdata.calendar.WebContent),
          "Calendar event entry <gCal:webContent> element must be an " +
          "instance of gdata.calendar.WebContent: %s" % web_content_element)
    self.assertEquals(expected_width, web_content_element.width)
    self.assertEquals(expected_height, web_content_element.height)
    self.assertEquals(expected_url, web_content_element.url)

  def assertValidWebContentGadget(self, expected_url=None, expected_width=None,
      expected_height=None, expected_pref_name=None, expected_pref_value=None, 
      web_content_element=None):
    """Asserts that the web content element is the correct type and contains
    the expected values"""

    self.assert_(isinstance(web_content_element, gdata.calendar.WebContent),
          "Calendar event entry <gCal:webContent> element must be an " +
          "instance of gdata.calendar.WebContent: %s" % web_content_element)
    self.assertEquals(expected_width, web_content_element.width)
    self.assertEquals(expected_height, web_content_element.height)
    self.assertEquals(expected_url, web_content_element.url)
    self.assertEquals(expected_pref_name, 
        web_content_element.gadget_pref[0].name)
    self.assertEquals(expected_pref_value, 
        web_content_element.gadget_pref[0].value)

  def testSampleCode(self):
    # From http://code.google.com/apis/calendar/gadgets/event/
    wc = gdata.calendar.WebContent()
    wc.url = 'http://www.thefreedictionary.com/_/WoD/wod-module.xml'
    wc.width = '300'
    wc.height = '136'
    wc.gadget_pref.append(gdata.calendar.WebContentGadgetPref(name='Days', value='1'))
    wc.gadget_pref.append(gdata.calendar.WebContentGadgetPref(name='Format', value='0'))
    wcl = gdata.calendar.WebContentLink()
    wcl.title = 'Word of the Day'
    wcl.href = 'http://www.thefreedictionary.com/favicon.ico'
    wcl.type = 'application/x-google-gadgets+xml'
    wcl.web_content = wc
    self.assertEqual(wcl.web_content.url,
                     'http://www.thefreedictionary.com/_/WoD/wod-module.xml')
    self.assertEqual(wcl.type, 'application/x-google-gadgets+xml')
    self.assertEqual(wcl.web_content.height, '136')


class ExtendedPropertyTest(unittest.TestCase):

  def testExtendedPropertyToAndFromXml(self):
    ep = gdata.calendar.ExtendedProperty(name='test')
    ep.value = 'val'
    xml_string = ep.ToString()
    ep2 = gdata.ExtendedPropertyFromString(xml_string)
    self.assertEquals(ep.name, ep2.name)
    self.assertEquals(ep.value, ep2.value)
      

if __name__ == '__main__':
  unittest.main()



__author__ = 'wiktorgworek@google.com (Wiktor Gworek)'

import wsgiref.handlers

import atom
import os
import cgi
import gdata.blogger.service

from oauth import OAuthDanceHandler, OAuthHandler, requiresOAuth
from google.appengine.ext import webapp
from google.appengine.ext.webapp import template


class MainHandler(OAuthHandler):
  """Main handler. If user is not logged in via OAuth it will display welcome
  page. In other case user's blogs on Blogger will be displayed."""

  def get(self):
    try:
      template_values = {'logged': self.client.has_access_token()}

      if template_values['logged']:
        feed = self.client.blogger.GetBlogFeed()
        blogs = []
        for entry in feed.entry:
          blogs.append({
            'id': entry.GetBlogId(),
            'title': entry.title.text,
            'link': entry.GetHtmlLink().href,
            'published': entry.published.text,
            'updated': entry.updated.text
          })
        template_values['blogs'] = blogs
    except gdata.service.RequestError, error:
      template_values['logged'] = False

    path = os.path.join(os.path.dirname(__file__), 'index.html')
    self.response.out.write(template.render(path, template_values))


class NewPostHandler(OAuthHandler):
  """Handles AJAX POST request to create a new post on a blog."""

  @requiresOAuth
  def post(self):
    entry = atom.Entry(content=atom.Content(text=self.request.get('body')))
    self.client.blogger.AddPost(entry, blog_id=self.request.get('id'))

def main():
  application = webapp.WSGIApplication([
    (r'/oauth/(.*)', OAuthDanceHandler),
    ('/new_post', NewPostHandler),
    ('/', MainHandler),
  ], debug=True)
  wsgiref.handlers.CGIHandler().run(application)

if __name__ == '__main__':
  main()



#!/usr/bin/python
#
# Copyright (C) 2008 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


__author__ = 'api.jscudder (Jeff Scudder)'


import unittest
from gdata import test_data
import gdata.blogger
import atom


class BlogEntryTest(unittest.TestCase):

  def testBlogEntryFromString(self):
    entry = gdata.blogger.BlogEntryFromString(test_data.BLOG_ENTRY)
    self.assertEquals(entry.GetBlogName(), 'blogName')
    self.assertEquals(entry.GetBlogId(), 'blogID')
    self.assertEquals(entry.title.text, 'Lizzy\'s Diary')

  def testBlogPostFeedFromString(self):
    feed = gdata.blogger.BlogPostFeedFromString(test_data.BLOG_POSTS_FEED)
    self.assertEquals(len(feed.entry), 1)
    self.assert_(isinstance(feed, gdata.blogger.BlogPostFeed))
    self.assert_(isinstance(feed.entry[0], gdata.blogger.BlogPostEntry))
    self.assertEquals(feed.entry[0].GetPostId(), 'postID')
    self.assertEquals(feed.entry[0].GetBlogId(), 'blogID')
    self.assertEquals(feed.entry[0].title.text, 'Quite disagreeable')

  def testCommentFeedFromString(self):
    feed = gdata.blogger.CommentFeedFromString(test_data.BLOG_COMMENTS_FEED)
    self.assertEquals(len(feed.entry), 1)
    self.assert_(isinstance(feed, gdata.blogger.CommentFeed))
    self.assert_(isinstance(feed.entry[0], gdata.blogger.CommentEntry))
    self.assertEquals(feed.entry[0].GetBlogId(), 'blogID')
    self.assertEquals(feed.entry[0].GetCommentId(), 'commentID')
    self.assertEquals(feed.entry[0].title.text, 'This is my first comment')
    self.assertEquals(feed.entry[0].in_reply_to.source, 
        'http://blogName.blogspot.com/feeds/posts/default/postID')
    self.assertEquals(feed.entry[0].in_reply_to.ref, 
        'tag:blogger.com,1999:blog-blogID.post-postID')
    self.assertEquals(feed.entry[0].in_reply_to.href, 
        'http://blogName.blogspot.com/2007/04/first-post.html')
    self.assertEquals(feed.entry[0].in_reply_to.type, 'text/html') 

  def testIdParsing(self):
    entry = gdata.blogger.BlogEntry()
    entry.id = atom.Id(
        text='tag:blogger.com,1999:user-146606542.blog-4023408167658848')
    self.assertEquals(entry.GetBlogId(), '4023408167658848')
    entry.id = atom.Id(text='tag:blogger.com,1999:blog-4023408167658848')
    self.assertEquals(entry.GetBlogId(), '4023408167658848')


class InReplyToTest(unittest.TestCase):

  def testToAndFromString(self):
    in_reply_to = gdata.blogger.InReplyTo(href='http://example.com/href',
        ref='http://example.com/ref', source='http://example.com/my_post',
        type='text/html')
    xml_string = str(in_reply_to)
    parsed = gdata.blogger.InReplyToFromString(xml_string)
    self.assertEquals(parsed.source, in_reply_to.source)
    self.assertEquals(parsed.href, in_reply_to.href)
    self.assertEquals(parsed.ref, in_reply_to.ref)
    self.assertEquals(parsed.type, in_reply_to.type)


class CommentEntryTest(unittest.TestCase):

  def testToAndFromString(self):
    comment = gdata.blogger.CommentEntry(content=atom.Content(text='Nifty!'),
        in_reply_to=gdata.blogger.InReplyTo(
            source='http://example.com/my_post'))
    parsed = gdata.blogger.CommentEntryFromString(str(comment))
    self.assertEquals(parsed.in_reply_to.source, comment.in_reply_to.source)
    self.assertEquals(parsed.content.text, comment.content.text)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python2.4
#
# Copyright 2012 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Google Apps marketplace sample app.

Demonstartes how to use provisoining data in marketplace apps.
"""

__author__ = 'Gunjan Sharma <gunjansharma@google.com>'

import logging
import os
import re
import urllib
from urlparse import urlparse
from django.utils import simplejson as json
from google.appengine.api import users
from google.appengine.ext import webapp
from google.appengine.ext.webapp import template
from google.appengine.ext.webapp import util
from appengine_utilities.sessions import Session
from gdata.apps.client import AppsClient
from gdata.apps.groups.client import GroupsProvisioningClient
from gdata.apps.organization.client import OrganizationUnitProvisioningClient
import gdata.auth

CONSUMER_KEY = '965697648820.apps.googleusercontent.com'
CONSUMER_SECRET = '3GBNP4EJykV7wq8tuN0LTFLr'


class TwoLeggedOauthTokenGenerator(webapp.RequestHandler):
  def Get2loToken(self):
    user = users.get_current_user()
    return gdata.gauth.TwoLeggedOAuthHmacToken(
        CONSUMER_KEY, CONSUMER_SECRET, user.email())


class MainHandler(TwoLeggedOauthTokenGenerator):
  """Handles initial get request and post request to '/' URL."""

  def get(self):
    """Handels the get request for the MainHandler.

    It checks if a the user is logged in and also that he belogs to the domain,
    if not redirects it to the login page else to the index.html page.
    """
    domain = self.request.get('domain')
    if not domain:
      self.response.out.write(
          'Missing required params. To use the app start with following URL: '
          'http://domain-mgmt.appspot.com?from=google&domain=yourdomain.com')
      return
    user = users.get_current_user()
    if user and self.CheckEmail(user):
      logging.debug('logged in user: %s', user.email())
      session = Session()
      session['domain'] = domain
    else:
      self.redirect('/_ah/login_required?' +
                    urllib.urlencode((self.request.str_params)))

    path = os.path.join(os.path.dirname(__file__), 'templates/index.html')
    self.response.out.write(template.render(path, {}))

  def CheckEmail(self, user):
    """Performs basic validation of the supplied email address.

    Args:
      user: A User object corresponding to logged in user.

    Returns:
      True if user is valid, False otherwise.
    """
    domain = urlparse(user.federated_identity()).hostname
    m = re.search('.*@' + domain, user.email())
    if m:
      return True
    else:
      return False

  def post(self):
    """Handels the get request for the MainHandler.

    Retrieves a list of all of the domain's users and sends it
    to the Client as a JSON object.
    """
    users_list = []
    session = Session()
    domain = session['domain']
    client = AppsClient(domain=domain)
    client.auth_token = self.Get2loToken()
    client.ssl = True
    feed = client.RetrieveAllUsers()
    for entry in feed.entry:
      users_list.append(entry.login.user_name)
    self.response.out.write(json.dumps(users_list))


class UserDetailsHandler(TwoLeggedOauthTokenGenerator):
  """Handles get request to '/getdetails' URL."""

  def get(self, username):
    """Handels the get request for the UserDetailsHandler.

    Sends groups, organization unit and nicknames for the user
    in a JSON object.

    Args:
      username: A string denoting the user's username.
    """
    session = Session()
    domain = session['domain']
    if not domain:
      self.redirect('/')
    details = {}
    details['groups'] = self.GetGroups(domain, username)
    details['orgunit'] = self.GetOrgunit(domain, username)
    details['nicknames'] = self.GetNicknames(domain, username)
    data = json.dumps(details)
    logging.debug('Sending data...')
    logging.debug(data)
    self.response.out.write(data)
    logging.debug('Data sent successfully')

  def GetGroups(self, domain, username):
    """Retrieves a list of groups for the given user.

    Args:
      domain: A string determining the user's domain.
      username: A string denoting the user's username.

    Returns:
      A list of dicts of groups with their name and ID if successful.
      Otherwise a list with single dict entry containing error message.
    """
    try:
      groups_client = GroupsProvisioningClient(domain=domain)
      groups_client.auth_token = self.Get2loToken()
      groups_client.ssl = True
      feed = groups_client.RetrieveGroups(username, True)
      groups = []
      for entry in feed.entry:
        group = {}
        group['name'] = entry.group_name
        group['id'] = entry.group_id
        groups.append(group)
      return groups
    except:
      return [{'name': 'An error occured while retriving Groups for the user',
               'id': 'An error occured while retriving Groups for the user'}]

  def GetOrgunit(self, domain, username):
    """Retrieves the Org Unit corresponding to the user.

    Args:
      domain: A string determining the user's domain.
      username: A string denoting the user's username.

    Returns:
      A dict of orgunit having its name and path if successful.
      Otherwise a dict entry containing error message.
    """
    try:
      ouclient = OrganizationUnitProvisioningClient(domain=domain)
      ouclient.auth_token = self.Get2loToken()
      ouclient.ssl = True
      customer_id = ouclient.RetrieveCustomerId().customer_id
      entry = ouclient.RetrieveOrgUser(customer_id, username + '@' + domain)
      oupath = entry.org_unit_path
      orgunit = {}
      if not oupath:
        orgunit['name'] = 'MAIN ORG UNIT'
        orgunit['path'] = '/'
        return orgunit
      entry = ouclient.RetrieveOrgUnit(customer_id, oupath)
      orgunit['name'] = entry.org_unit_name
      orgunit['path'] = entry.org_unit_path
      return orgunit
    except:
      return {'name': 'An error occured while retriving OrgUnit for the user.',
              'path': 'An error occured while retriving OrgUnit for the user.'}

  def GetNicknames(self, domain, username):
    """Retrieves the list of all the nicknames for the user.

    Args:
      domain: A string determining the user's domain.
      username: A string denoting the user's username.

    Returns:
      A list of user's nicknames if successful.
      Otherwise a list with a single entry containing error message.
    """
    try:
      client = AppsClient(domain=domain)
      client.auth_token = self.Get2loToken()
      client.ssl = True
      feed = client.RetrieveNicknames(username)
      nicknames = []
      for entry in feed.entry:
        nicknames.append(entry.nickname.name)
      return nicknames
    except:
      return ['An error occured while retriving Nicknames for the user.']


class OpenIDHandler(webapp.RequestHandler):
  def get(self):
    """Begins the OpenID flow for the supplied domain."""
    domain = self.request.get('domain')
    self.redirect(users.create_login_url(
        dest_url='https://domain-mgmt.appspot.com?domain=' + domain,
        _auth_domain=None,
        federated_identity=domain))


def main():
  application = webapp.WSGIApplication([('/', MainHandler),
                                        ('/getdetails/(.*)',
                                         UserDetailsHandler),
                                        ('/_ah/login_required', OpenIDHandler)],
                                       debug=True)
  util.run_wsgi_app(application)


if __name__ == '__main__':
  main()



#!/usr/bin/python
#
# Copyright (C) 2007 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#
# This sample uses the Google Spreadsheets data API and the Google 
# Calendar data API. The script pulls a list of birthdays from a 
# Google Spreadsheet and inserts them as webContent events in the
# user's Google Calendar. 
# 
# The script expects a certain format in the spreadsheet: Name,
# Birthday, Photo URL, and Edit URL as headers. Expected format 
# of the birthday is: MM/DD. Edit URL is to be left blank by the
# user - the script uses this column to determine whether to insert
# a new event or to update an event at the URL. 
# 
# See the spreadsheet below for an example:
# http://spreadsheets.google.com/pub?key=pfMX-JDVnx47J0DxqssIQHg
#


__author__ = 'api.stephaniel@google.com (Stephanie Liu)'

try:
  from xml.etree import ElementTree # for Python 2.5 users
except:  
  from elementtree import ElementTree

import gdata.spreadsheet.service
import gdata.calendar.service
import gdata.calendar
import gdata.service
import atom.service
import gdata.spreadsheet
import atom
import string
import time
import datetime
import getopt
import getpass
import sys


class BirthdaySample:
  # CONSTANTS: Expected column headers: name, birthday, photourl, editurl &
  # default calendar reminder set to 2 days
  NAME = "name"
  BIRTHDAY = "birthday"
  PHOTO_URL = "photourl"
  EDIT_URL = "editurl"
  REMINDER = 60 * 24 * 2 # minutes

  def __init__(self, email, password):
    """ Initializes spreadsheet and calendar clients.
        
        Creates SpreadsheetsService and CalendarService objects and 
        authenticates to each with ClientLogin. For more information
        about ClientLogin authentication: 
        http://code.google.com/apis/accounts/AuthForInstalledApps.html

        Args:
          email: string
          password: string
    """

    self.s_client = gdata.spreadsheet.service.SpreadsheetsService()
    self.s_client.email = email
    self.s_client.password = password
    self.s_client.source = 'exampleCo-birthdaySample-1'
    self.s_client.ProgrammaticLogin()

    self.c_client = gdata.calendar.service.CalendarService()
    self.c_client.email = email
    self.c_client.password = password
    self.c_client.source = 'exampleCo-birthdaySample-1'
    self.c_client.ProgrammaticLogin()
  
  def _PrintFeed(self, feed):
    """ Prints out Spreadsheet feeds in human readable format.
      
        Generic function taken from spreadsheetsExample.py.          

        Args:
          feed: SpreadsheetsCellsFeed, SpreadsheetsListFeed,
            SpreadsheetsWorksheetsFeed, or SpreadsheetsSpreadsheetsFeed
    """
    for i, entry in enumerate(feed.entry):
      if isinstance(feed, gdata.spreadsheet.SpreadsheetsCellsFeed):
        print '%s %s\n' % (entry.title.text, entry.content.text)
      elif isinstance(feed, gdata.spreadsheet.SpreadsheetsListFeed):
        print '%s %s %s\n' % (i, entry.title.text, entry.content.text)
      else:
        print '%s %s\n' % (i, entry.title.text)

  def _PromptForSpreadsheet(self):
    """ Prompts user to select spreadsheet.
     
        Gets and displays titles of all spreadsheets for user to 
        select. Generic function taken from spreadsheetsExample.py.

        Args:
          none

        Returns:
           spreadsheet ID that the user selected: string
    """

    feed = self.s_client.GetSpreadsheetsFeed()
    self._PrintFeed(feed)
    input = raw_input('\nSelection: ')
    
    # extract and return the spreadsheet ID
    return feed.entry[string.atoi(input)].id.text.rsplit('/', 1)[1]

  def _PromptForWorksheet(self, key):
    """ Prompts user to select desired worksheet.

        Gets and displays titles of all worksheets for user to
        select. Generic function taken from spreadsheetsExample.py.

        Args:
          key: string

        Returns:
           the worksheet ID that the user selected: string
    """

    feed = self.s_client.GetWorksheetsFeed(key)
    self._PrintFeed(feed)
    input = raw_input('\nSelection: ')

    # extract and return the worksheet ID
    return feed.entry[string.atoi(input)].id.text.rsplit('/', 1)[1]

  def _AddReminder(self, event, minutes):
    """ Adds a reminder to a calendar event. 

        This function sets the reminder attribute of the CalendarEventEntry.
        The script sets it to 2 days by default, and this value is not 
        settable by the user. However, it can easily be changed to take this
        option.

        Args:
          event: CalendarEventEntry
          minutes: int
 
        Returns:
          the updated event: CalendarEventEntry
    """

    for a_when in event.when:
      if len(a_when.reminder) > 0:
        a_when.reminder[0].minutes = minutes
      else:
        a_when.reminder.append(gdata.calendar.Reminder(minutes=minutes))

    return self.c_client.UpdateEvent(event.GetEditLink().href, event)

  def _CreateBirthdayWebContentEvent(self, name, birthday, photo_url):
    """ Create the birthday web content event.

        This function creates and populates a CalendarEventEntry. webContent
        specific attributes are set. To learn more about the webContent
        format:

        http://www.google.com/support/calendar/bin/answer.py?answer=48528

        Args:
          name: string
          birthday: string - expected format (MM/DD)
          photo_url: string     

        Returns:
           the webContent CalendarEventEntry
    """   

    title = "%s's Birthday!" % name
    content = "It's %s's Birthday!" % name
    month = string.atoi(birthday.split("/")[0])
    day = string.atoi(birthday.split("/")[1])

    # Get current year
    year = time.ctime()[-4:]
    year = string.atoi(year)
    
    # Calculate the "end date" for the all day event
    start_time = datetime.date(year, month, day)
    one_day = datetime.timedelta(days=1)
    end_time = start_time + one_day

    start_time_str = start_time.strftime("%Y-%m-%d")
    end_time_str = end_time.strftime("%Y-%m-%d")

    # Create yearly recurrence rule
    recurrence_data = ("DTSTART;VALUE=DATE:%s\r\n"
        "DTEND;VALUE=DATE:%s\r\n"
        "RRULE:FREQ=YEARLY;WKST=SU\r\n" % 
        (start_time.strftime("%Y%m%d"), end_time.strftime("%Y%m%d")))
 
    web_rel = "http://schemas.google.com/gCal/2005/webContent"
    icon_href = "http://www.perstephanie.com/images/birthdayicon.gif"
    icon_type = "image/gif"
    extension_text = (
        'gCal:webContent xmlns:gCal="http://schemas.google.com/gCal/2005"'
        ' url="%s" width="300" height="225"' % (photo_url))

    event = gdata.calendar.CalendarEventEntry()
    event.title = atom.Title(text=title)
    event.content = atom.Content(text=content)
    event.recurrence = gdata.calendar.Recurrence(text=recurrence_data)
    event.when.append(gdata.calendar.When(start_time=start_time_str, 
        end_time=end_time_str))

    # Adding the webContent specific XML 
    event.link.append(atom.Link(rel=web_rel, title=title, href=icon_href, 
        link_type=icon_type))
    event.link[0].extension_elements.append(
        atom.ExtensionElement(extension_text))
 
    return event

  def _InsertBirthdayWebContentEvent(self, event):
    """ Insert event into the authenticated user's calendar. 

        Args:
          event: CalendarEventEntry

        Returns:
           the newly created CalendarEventEntry 
    """

    edit_uri = '/calendar/feeds/default/private/full'
    return self.c_client.InsertEvent(event, edit_uri)

  def Run(self):
    """ Run sample.
 
        TODO: add exception handling 

        Args:
          none
    """
     
    key_id = self._PromptForSpreadsheet()
    wksht_id = self._PromptForWorksheet(key_id)
 
    feed = self.s_client.GetListFeed(key_id, wksht_id) 

    found_name = False
    found_birthday = False
    found_photourl = False
    found_editurl = False 

    # Check to make sure all headers are present
    # Need to find at least one instance of name, birthday, photourl
    # editurl 
    if len(feed.entry) > 0:
      for name, custom in feed.entry[0].custom.iteritems():
        if custom.column == self.NAME:
          found_name = True
        elif custom.column == self.BIRTHDAY:
          found_birthday = True
        elif custom.column == self.PHOTO_URL:
          found_photourl = True
        elif custom.column == self.EDIT_URL:
          found_editurl = True

    if not found_name and found_birthday and found_photourl and found_editurl:
      print  ("ERROR - Unexpected number of column headers. Should have: %s,"
             " %s, %s, and %s." % (self.NAME, self.BIRTHDAY, self.PHOTO_URL, 
             self.EDIT_URL)) 
      sys.exit(1)

    # For every row in the spreadsheet, grab all the data and either insert
    # a new event into the calendar, or update the existing event

    # Create dict to represent the row data to update edit link back to
    # Spreadsheet
     
    for entry in feed.entry:
      d = {} 
      input_valid = True
      
      for name, custom in entry.custom.iteritems():
        d[custom.column] = custom.text

      month = int(d[self.BIRTHDAY].split("/")[0]) 
      day = int(d[self.BIRTHDAY].split("/")[1])
      
      # Some input checking. Script will allow the insert to continue with
      # a missing name value.
      if d[self.NAME] is None:
        d[self.NAME] = " "
      if d[self.PHOTO_URL] is None:
        input_valid = False
      if d[self.BIRTHDAY] is None:
        input_valid = False
      elif not 1 <= month <= 12 or not 1 <= day <= 31:
        input_valid = False  

      if d[self.EDIT_URL] is None and input_valid:
        event = self._CreateBirthdayWebContentEvent(d[self.NAME], 
            d[self.BIRTHDAY], d[self.PHOTO_URL])
        event = self._InsertBirthdayWebContentEvent(event)
        event = self._AddReminder(event, self.REMINDER)
	print "Added %s's birthday!" % d[self.NAME]
      elif input_valid: # Event already exists
        edit_link = d[self.EDIT_URL]
        event = self._CreateBirthdayWebContentEvent(d[self.NAME], 
            d[self.BIRTHDAY], d[self.PHOTO_URL])          
        event = self.c_client.UpdateEvent(edit_link, event)
        event = self._AddReminder(event, self.REMINDER)
        print "Updated %s's birthday!" % d[self.NAME]
      
      if input_valid:
        d[self.EDIT_URL] = event.GetEditLink().href
        self.s_client.UpdateRow(entry, d)
      else:
        print "Warning - Skipping row, missing valid input." 

def main():
  email = raw_input("Please enter your email: ")
  password = getpass.getpass("Please enter your password: ")
  
  sample = BirthdaySample(email, password)
  sample.Run()


if __name__ == '__main__':
  main()




#!/usr/bin/env python
#
# Copyright (C) 2009 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# This module is used for version 2 of the Google Data APIs.


__author__ = 'j.s@google.com (Jeff Scudder)'


import unittest
import gdata.test_config as conf
import gdata.data
import gdata.acl.data
import gdata.analytics.data
import gdata.dublincore.data
import gdata.books.data
import gdata.calendar.data
import gdata.geo.data
import gdata.finance.data
import gdata.notebook.data
import gdata.media.data
import gdata.youtube.data
import gdata.webmastertools.data
import gdata.contacts.data
import gdata.opensearch.data


class DataSmokeTest(unittest.TestCase):

  def test_check_all_data_classes(self):
    conf.check_data_classes(self, (
        gdata.data.TotalResults, gdata.data.StartIndex,
        gdata.data.ItemsPerPage, gdata.data.ExtendedProperty,
        gdata.data.GDEntry, gdata.data.GDFeed, gdata.data.BatchId,
        gdata.data.BatchOperation, gdata.data.BatchStatus,
        gdata.data.BatchEntry, gdata.data.BatchInterrupted,
        gdata.data.BatchFeed, gdata.data.EntryLink, gdata.data.FeedLink,
        gdata.data.AdditionalName, gdata.data.Comments, gdata.data.Country,
        gdata.data.Email, gdata.data.FamilyName, gdata.data.Im,
        gdata.data.GivenName, gdata.data.NamePrefix, gdata.data.NameSuffix,
        gdata.data.FullName, gdata.data.Name, gdata.data.OrgDepartment,
        gdata.data.OrgName, gdata.data.OrgSymbol, gdata.data.OrgTitle,
        gdata.data.Organization, gdata.data.When, gdata.data.Who,
        gdata.data.OriginalEvent, gdata.data.PhoneNumber,
        gdata.data.PostalAddress, gdata.data.Rating, gdata.data.Recurrence,
        gdata.data.RecurrenceException, gdata.data.Reminder,
        gdata.data.Agent, gdata.data.HouseName, gdata.data.Street,
        gdata.data.PoBox, gdata.data.Neighborhood, gdata.data.City,
        gdata.data.Subregion, gdata.data.Region, gdata.data.Postcode,
        gdata.data.Country, gdata.data.FormattedAddress,
        gdata.data.StructuredPostalAddress, gdata.data.Where,
        gdata.data.AttendeeType, gdata.data.AttendeeStatus,
        gdata.data.Deleted, gdata.data.Money,
        gdata.acl.data.AclRole, gdata.acl.data.AclScope,
        gdata.acl.data.AclWithKey,
        gdata.acl.data.AclEntry, gdata.acl.data.AclFeed,
        gdata.analytics.data.Dimension,
        gdata.analytics.data.EndDate,
        gdata.analytics.data.Metric,
        gdata.analytics.data.Aggregates,
        gdata.analytics.data.DataEntry,
        gdata.analytics.data.Property,
        gdata.analytics.data.StartDate,
        gdata.analytics.data.TableId,
        gdata.analytics.data.AccountEntry,
        gdata.analytics.data.TableName,
        gdata.analytics.data.DataSource,
        gdata.analytics.data.AccountFeed,
        gdata.analytics.data.DataFeed,
        gdata.dublincore.data.Creator,
        gdata.dublincore.data.Date,
        gdata.dublincore.data.Description,
        gdata.dublincore.data.Format,
        gdata.dublincore.data.Identifier,
        gdata.dublincore.data.Language,
        gdata.dublincore.data.Publisher,
        gdata.dublincore.data.Rights,
        gdata.dublincore.data.Subject,
        gdata.dublincore.data.Title,
        gdata.books.data.CollectionEntry,
        gdata.books.data.CollectionFeed,
        gdata.books.data.Embeddability,
        gdata.books.data.OpenAccess,
        gdata.books.data.Review,
        gdata.books.data.Viewability,
        gdata.books.data.VolumeEntry,
        gdata.books.data.VolumeFeed,
        gdata.calendar.data.AccessLevelProperty,
        gdata.calendar.data.AllowGSync2Property,
        gdata.calendar.data.AllowGSyncProperty,
        gdata.calendar.data.AnyoneCanAddSelfProperty,
        gdata.calendar.data.CalendarAclRole,
        gdata.calendar.data.CalendarCommentEntry,
        gdata.calendar.data.CalendarCommentFeed,
        gdata.calendar.data.CalendarComments,
        gdata.calendar.data.CalendarExtendedProperty,
        gdata.calendar.data.CalendarWhere,
        gdata.calendar.data.ColorProperty,
        gdata.calendar.data.GuestsCanInviteOthersProperty,
        gdata.calendar.data.GuestsCanModifyProperty,
        gdata.calendar.data.GuestsCanSeeGuestsProperty,
        gdata.calendar.data.HiddenProperty,
        gdata.calendar.data.IcalUIDProperty,
        gdata.calendar.data.OverrideNameProperty,
        gdata.calendar.data.PrivateCopyProperty,
        gdata.calendar.data.QuickAddProperty,
        gdata.calendar.data.ResourceProperty,
        gdata.calendar.data.EventWho,
        gdata.calendar.data.SelectedProperty,
        gdata.calendar.data.SendAclNotificationsProperty,
        gdata.calendar.data.CalendarAclEntry,
        gdata.calendar.data.CalendarAclFeed,
        gdata.calendar.data.SendEventNotificationsProperty,
        gdata.calendar.data.SequenceNumberProperty,
        gdata.calendar.data.CalendarRecurrenceExceptionEntry,
        gdata.calendar.data.CalendarRecurrenceException,
        gdata.calendar.data.SettingsProperty,
        gdata.calendar.data.SettingsEntry,
        gdata.calendar.data.CalendarSettingsFeed,
        gdata.calendar.data.SuppressReplyNotificationsProperty,
        gdata.calendar.data.SyncEventProperty,
        gdata.calendar.data.CalendarEventEntry,
        gdata.calendar.data.TimeZoneProperty,
        gdata.calendar.data.TimesCleanedProperty,
        gdata.calendar.data.CalendarEntry,
        gdata.calendar.data.CalendarEventFeed,
        gdata.calendar.data.CalendarFeed,
        gdata.calendar.data.WebContentGadgetPref,
        gdata.calendar.data.WebContent,
        gdata.finance.data.Commission,
        gdata.finance.data.CostBasis,
        gdata.finance.data.DaysGain,
        gdata.finance.data.Gain,
        gdata.finance.data.MarketValue,
        gdata.finance.data.PortfolioData,
        gdata.finance.data.PortfolioEntry,
        gdata.finance.data.PortfolioFeed,
        gdata.finance.data.PositionData,
        gdata.finance.data.Price,
        gdata.finance.data.Symbol,
        gdata.finance.data.PositionEntry,
        gdata.finance.data.PositionFeed,
        gdata.finance.data.TransactionData,
        gdata.finance.data.TransactionEntry,
        gdata.finance.data.TransactionFeed,
        gdata.notebook.data.ComesAfter,
        gdata.notebook.data.NoteEntry,
        gdata.notebook.data.NotebookFeed,
        gdata.notebook.data.NotebookListEntry,
        gdata.notebook.data.NotebookListFeed,
        gdata.youtube.data.ComplaintEntry,
        gdata.youtube.data.ComplaintFeed,
        gdata.youtube.data.RatingEntry,
        gdata.youtube.data.RatingFeed,
        gdata.youtube.data.YouTubeMediaContent,
        gdata.youtube.data.YtAge,
        gdata.youtube.data.YtBooks,
        gdata.youtube.data.YtCompany,
        gdata.youtube.data.YtDescription,
        gdata.youtube.data.YtDuration,
        gdata.youtube.data.YtFirstName,
        gdata.youtube.data.YtGender,
        gdata.youtube.data.YtHobbies,
        gdata.youtube.data.YtHometown,
        gdata.youtube.data.YtLastName,
        gdata.youtube.data.YtLocation,
        gdata.youtube.data.YtMovies,
        gdata.youtube.data.YtMusic,
        gdata.youtube.data.YtNoEmbed,
        gdata.youtube.data.YtOccupation,
        gdata.youtube.data.YtPlaylistId,
        gdata.youtube.data.YtPosition,
        gdata.youtube.data.YtPrivate,
        gdata.youtube.data.YtQueryString,
        gdata.youtube.data.YtRacy,
        gdata.youtube.data.YtRecorded,
        gdata.youtube.data.YtRelationship,
        gdata.youtube.data.YtSchool,
        gdata.youtube.data.YtStatistics,
        gdata.youtube.data.YtStatus,
        gdata.youtube.data.YtUserProfileStatistics,
        gdata.youtube.data.YtUsername,
        gdata.youtube.data.FriendEntry,
        gdata.youtube.data.FriendFeed,
        gdata.youtube.data.YtVideoStatistics,
        gdata.youtube.data.ChannelEntry,
        gdata.youtube.data.ChannelFeed,
        gdata.youtube.data.FavoriteEntry,
        gdata.youtube.data.FavoriteFeed,
        gdata.youtube.data.YouTubeMediaCredit,
        gdata.youtube.data.YouTubeMediaRating,
        gdata.youtube.data.YtAboutMe,
        gdata.youtube.data.UserProfileEntry,
        gdata.youtube.data.UserProfileFeed,
        gdata.youtube.data.YtAspectRatio,
        gdata.youtube.data.YtBasePublicationState,
        gdata.youtube.data.YtPublicationState,
        gdata.youtube.data.YouTubeAppControl,
        gdata.youtube.data.YtCaptionPublicationState,
        gdata.youtube.data.YouTubeCaptionAppControl,
        gdata.youtube.data.CaptionTrackEntry,
        gdata.youtube.data.CaptionTrackFeed,
        gdata.youtube.data.YtCountHint,
        gdata.youtube.data.PlaylistLinkEntry,
        gdata.youtube.data.PlaylistLinkFeed,
        gdata.youtube.data.YtModerationStatus,
        gdata.youtube.data.YtPlaylistTitle,
        gdata.youtube.data.SubscriptionEntry,
        gdata.youtube.data.SubscriptionFeed,
        gdata.youtube.data.YtSpam,
        gdata.youtube.data.CommentEntry,
        gdata.youtube.data.CommentFeed,
        gdata.youtube.data.YtUploaded,
        gdata.youtube.data.YtVideoId,
        gdata.youtube.data.YouTubeMediaGroup,
        gdata.youtube.data.VideoEntryBase,
        gdata.youtube.data.PlaylistEntry,
        gdata.youtube.data.PlaylistFeed,
        gdata.youtube.data.VideoEntry,
        gdata.youtube.data.VideoFeed,
        gdata.youtube.data.VideoMessageEntry,
        gdata.youtube.data.VideoMessageFeed,
        gdata.youtube.data.UserEventEntry,
        gdata.youtube.data.UserEventFeed,
        gdata.youtube.data.VideoModerationEntry,
        gdata.youtube.data.VideoModerationFeed,
        gdata.media.data.MediaCategory,
        gdata.media.data.MediaCopyright,
        gdata.media.data.MediaCredit,
        gdata.media.data.MediaDescription,
        gdata.media.data.MediaHash,
        gdata.media.data.MediaKeywords,
        gdata.media.data.MediaPlayer,
        gdata.media.data.MediaRating,
        gdata.media.data.MediaRestriction,
        gdata.media.data.MediaText,
        gdata.media.data.MediaThumbnail,
        gdata.media.data.MediaTitle,
        gdata.media.data.MediaContent,
        gdata.media.data.MediaGroup,
        gdata.webmastertools.data.CrawlIssueCrawlType,
        gdata.webmastertools.data.CrawlIssueDateDetected,
        gdata.webmastertools.data.CrawlIssueDetail,
        gdata.webmastertools.data.CrawlIssueIssueType,
        gdata.webmastertools.data.CrawlIssueLinkedFromUrl,
        gdata.webmastertools.data.CrawlIssueUrl,
        gdata.webmastertools.data.CrawlIssueEntry,
        gdata.webmastertools.data.CrawlIssuesFeed,
        gdata.webmastertools.data.Indexed,
        gdata.webmastertools.data.Keyword,
        gdata.webmastertools.data.KeywordEntry,
        gdata.webmastertools.data.KeywordsFeed,
        gdata.webmastertools.data.LastCrawled,
        gdata.webmastertools.data.MessageBody,
        gdata.webmastertools.data.MessageDate,
        gdata.webmastertools.data.MessageLanguage,
        gdata.webmastertools.data.MessageRead,
        gdata.webmastertools.data.MessageSubject,
        gdata.webmastertools.data.SiteId,
        gdata.webmastertools.data.MessageEntry,
        gdata.webmastertools.data.MessagesFeed,
        gdata.webmastertools.data.SitemapEntry,
        gdata.webmastertools.data.SitemapMobileMarkupLanguage,
        gdata.webmastertools.data.SitemapMobile,
        gdata.webmastertools.data.SitemapNewsPublicationLabel,
        gdata.webmastertools.data.SitemapNews,
        gdata.webmastertools.data.SitemapType,
        gdata.webmastertools.data.SitemapUrlCount,
        gdata.webmastertools.data.SitemapsFeed,
        gdata.webmastertools.data.VerificationMethod,
        gdata.webmastertools.data.Verified,
        gdata.webmastertools.data.SiteEntry,
        gdata.webmastertools.data.SitesFeed,
        gdata.contacts.data.BillingInformation,
        gdata.contacts.data.Birthday,
        gdata.contacts.data.CalendarLink,
        gdata.contacts.data.DirectoryServer,
        gdata.contacts.data.Event,
        gdata.contacts.data.ExternalId,
        gdata.contacts.data.Gender,
        gdata.contacts.data.Hobby,
        gdata.contacts.data.Initials,
        gdata.contacts.data.Jot,
        gdata.contacts.data.Language,
        gdata.contacts.data.MaidenName,
        gdata.contacts.data.Mileage,
        gdata.contacts.data.NickName,
        gdata.contacts.data.Occupation,
        gdata.contacts.data.Priority,
        gdata.contacts.data.Relation,
        gdata.contacts.data.Sensitivity,
        gdata.contacts.data.UserDefinedField,
        gdata.contacts.data.Website,
        gdata.contacts.data.HouseName,
        gdata.contacts.data.Street,
        gdata.contacts.data.POBox,
        gdata.contacts.data.Neighborhood,
        gdata.contacts.data.City,
        gdata.contacts.data.SubRegion,
        gdata.contacts.data.Region,
        gdata.contacts.data.PostalCode,
        gdata.contacts.data.Country,
        gdata.contacts.data.PersonEntry,
        gdata.contacts.data.Deleted,
        gdata.contacts.data.GroupMembershipInfo,
        gdata.contacts.data.ContactEntry,
        gdata.contacts.data.ContactsFeed,
        gdata.contacts.data.SystemGroup,
        gdata.contacts.data.GroupEntry,
        gdata.contacts.data.GroupsFeed,
        gdata.contacts.data.ProfileEntry,
        gdata.contacts.data.ProfilesFeed,
        gdata.opensearch.data.ItemsPerPage,
        gdata.opensearch.data.StartIndex,
        gdata.opensearch.data.TotalResults,
    ))


def suite():
  return conf.build_suite([DataSmokeTest])


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

__author__ = 'samuel.cyprian@gmail.com (Samuel Cyprian)'

import unittest
from gdata import oauth, test_config

HTTP_METHOD_POST = 'POST'
VERSION = '1.0'


class OauthUtilsTest(unittest.TestCase):
  def test_build_authenticate_header(self):
    self.assertEqual(oauth.build_authenticate_header(), 
                     {'WWW-Authenticate' :'OAuth realm=""'})
    self.assertEqual(oauth.build_authenticate_header('foo'), 
                     {'WWW-Authenticate': 'OAuth realm="foo"'})
    
  def test_escape(self):
    #Special cases
    self.assertEqual(oauth.escape('~'), '~')
    self.assertEqual(oauth.escape('/'), '%2F')
    self.assertEqual(oauth.escape('+'), '%2B')
    self.assertEqual(oauth.escape(' '), '%20')
    self.assertEqual(oauth.escape('Peter Strömberg'), 
                       'Peter%20Str%C3%B6mberg')
  
  def test_generate_timestamp(self):
      self.assertTrue(oauth.generate_timestamp()>0)
      self.assertTrue(type(oauth.generate_timestamp()) is type(0))
      
  def test_generate_nonce(self):
    DEFAULT_NONCE_LENGTH = 8
    self.assertTrue(len(oauth.generate_nonce()) is DEFAULT_NONCE_LENGTH)
    self.assertTrue(type(oauth.generate_nonce()) is type(''))
      
class OAuthConsumerTest(unittest.TestCase):
  def setUp(self):
    self.key = 'key'
    self.secret = 'secret'
    self.consumer = oauth.OAuthConsumer(self.key, self.secret)
  
  def test_OAuthConsumer_attr_key(self):
    self.assertEqual(self.consumer.key, self.key)

  def test_OAuthConsumer_attr_secret(self):
    self.assertEqual(self.consumer.secret, self.secret)
     
class OAuthTokenTest(unittest.TestCase):
  def setUp(self):
    self.key = 'key'
    self.secret = 'secret'
    self.token = oauth.OAuthToken(self.key, self.secret)
        
  def test_OAuthToken_attr_key(self):
    self.assertEqual(self.token.key, self.key)

  def test_OAuthToken_attr_secret(self):
    self.assertEqual(self.token.secret, self.secret)
      
  def test_to_string(self):
    self.assertEqual(self.token.to_string(), 
                     'oauth_token_secret=secret&oauth_token=key')
    t = oauth.OAuthToken('+', '%')
    self.assertEqual(t.to_string(), 
                     'oauth_token_secret=%25&oauth_token=%2B')

  def test_from_string(self):
    s = 'oauth_token_secret=secret&oauth_token=key'
    t = oauth.OAuthToken.from_string(s)
    self.assertEqual(t.key, 'key')
    self.assertEqual(t.secret, 'secret')
    t = oauth.OAuthToken.from_string('oauth_token_secret=%25&oauth_token=%2B')
    self.assertEqual(t.key, '+')
    self.assertEqual(t.secret, '%')

  def test___str__(self):
    self.assertEqual(str(self.token), 
                     'oauth_token_secret=secret&oauth_token=key')
    t = oauth.OAuthToken('+', '%')
    self.assertEqual(str(t), 'oauth_token_secret=%25&oauth_token=%2B')
        
class OAuthParameters(object):
  CONSUMER_KEY = 'oauth_consumer_key'
  TOKEN = 'oauth_token'
  SIGNATURE_METHOD = 'oauth_signature_method'
  SIGNATURE = 'oauth_signature'
  TIMESTAMP = 'oauth_timestamp'
  NONCE = 'oauth_nonce'
  VERSION = 'oauth_version'
  CALLBACK = 'oauth_callback'
  
  ALL_PARAMETERS = (CONSUMER_KEY,
                    TOKEN,
                    SIGNATURE_METHOD,
                    SIGNATURE,
                    TIMESTAMP,
                    NONCE,
                    VERSION)
    
class OAuthTest(unittest.TestCase):
  def setUp(self):
    self.consumer = oauth.OAuthConsumer('a56b5ff0a637ab283d1d8e32ced37a9c', 
                                        '9a3248210c84b264b56b98c0b872bc8a')
    self.token = oauth.OAuthToken('5b2cafbf20b11bace53b29e37d8a673d', 
                                  '3f71254637df2002d8819458ae4f6c51')
    self.http_url = 'http://dev.alicehub.com/server/api/newsfeed/update/'
    
    self.http_method = HTTP_METHOD_POST

class OAuthRequestTest(OAuthTest):
  def setUp(self):
    super(OAuthRequestTest, self).setUp()
    self.signature_method = oauth.OAuthSignatureMethod_HMAC_SHA1()
    
    self.non_oauth_param_message = 'message'
    self.non_oauth_param_context_id = 'context_id'
    self.parameters = {OAuthParameters.CONSUMER_KEY:self.consumer.key,
                       OAuthParameters.TOKEN: self.token.key,
                       OAuthParameters.SIGNATURE_METHOD: 'HMAC-SHA1',
                       OAuthParameters.SIGNATURE:
                       '947ysBZiMn6FGZ11AW06Ioco4mo=',
                       OAuthParameters.TIMESTAMP: '1278573584',
                       OAuthParameters.NONCE: '1770704051',
                       OAuthParameters.VERSION: VERSION,
                       self.non_oauth_param_message:'hey',
                       self.non_oauth_param_context_id:'',}
    oauth_params_string = """ 
      oauth_nonce="1770704051", 
      oauth_timestamp="1278573584", 
      oauth_consumer_key="a56b5ff0a637ab283d1d8e32ced37a9c", 
      oauth_signature_method="HMAC-SHA1", 
      oauth_version="1.0", 
      oauth_token="5b2cafbf20b11bace53b29e37d8a673d", 
      oauth_signature="947ysBZiMn6FGZ11AW06Ioco4mo%3D"
    """
    self.oauth_header_with_realm = {'Authorization': """OAuth 
    realm="http://example.com", %s """ % oauth_params_string}
    self.oauth_header_without_realm = {'Authorization': 'OAuth %s' 
                                       % oauth_params_string}
    
    
    self.additional_param = 'foo'
    self.additional_value = 'bar'
    
    self.oauth_request = oauth.OAuthRequest(self.http_method, 
                                            self.http_url, 
                                            self.parameters)
      
  def test_set_parameter(self):
    self.oauth_request.set_parameter(self.additional_param, 
                                     self.additional_value)
    self.assertEqual(self.oauth_request.get_parameter(self.additional_param), 
                     self.additional_value)
      
  def test_get_parameter(self):
    self.assertRaises(oauth.OAuthError, 
                      self.oauth_request.get_parameter, 
                      self.additional_param)
    self.oauth_request.set_parameter(self.additional_param, 
                                     self.additional_value)
    self.assertEqual(self.oauth_request.get_parameter(self.additional_param), 
                     self.additional_value)
      
  def test__get_timestamp_nonce(self):
    self.assertEqual(self.oauth_request._get_timestamp_nonce(), 
                     (self.parameters[OAuthParameters.TIMESTAMP],
                      self.parameters[OAuthParameters.NONCE]))
    
  def test_get_nonoauth_parameters(self):
    non_oauth_params = self.oauth_request.get_nonoauth_parameters()
    self.assertTrue(non_oauth_params.has_key(self.non_oauth_param_message))
    self.assertFalse(non_oauth_params.has_key(OAuthParameters.CONSUMER_KEY))
      
  def test_to_header(self):
    realm = 'google'
    header_without_realm = self.oauth_request.to_header()\
    .get('Authorization')
    header_with_realm = self.oauth_request.to_header(realm)\
    .get('Authorization')
    self.assertTrue(header_with_realm.find(realm))
    for k in OAuthParameters.ALL_PARAMETERS:
      self.assertTrue(header_without_realm.find(k) > -1)
      self.assertTrue(header_with_realm.find(k) > -1)

  def check_for_params_in_string(self, params, s):
    for k, v in params.iteritems():
      self.assertTrue(s.find(oauth.escape(k)) > -1)
      self.assertTrue(s.find(oauth.escape(v)) > -1)
          
  def test_to_postdata(self):
    post_data = self.oauth_request.to_postdata()
    self.check_for_params_in_string(self.parameters, post_data)
            
  def test_to_url(self):
    GET_url = self.oauth_request.to_url()
    self.assertTrue(GET_url\
                    .find(self.oauth_request.get_normalized_http_url()) > -1)
    self.assertTrue(GET_url.find('?') > -1)
    self.check_for_params_in_string(self.parameters, GET_url)
      
  def test_get_normalized_parameters(self):
    _params = self.parameters.copy()
    normalized_params = self.oauth_request.get_normalized_parameters()
    self.assertFalse(normalized_params\
                     .find(OAuthParameters.SIGNATURE + '=') > -1)
    self.assertTrue(self.parameters.get(OAuthParameters.SIGNATURE) is None)
    
    key_values = [tuple(kv.split('=')) for kv in normalized_params.split('&')]
    del _params[OAuthParameters.SIGNATURE]
    expected_key_values = _params.items()
    expected_key_values.sort()
    
    for k, v in expected_key_values:
      self.assertTrue(expected_key_values.index((k,v))\
                      is key_values.index((oauth.escape(k), oauth.escape(v))))
      
  def test_get_normalized_http_method(self):
    lower_case_http_method = HTTP_METHOD_POST.lower()
    self.oauth_request.http_method = lower_case_http_method
    self.assertEqual(self.oauth_request.get_normalized_http_method(), 
                     lower_case_http_method.upper())
        
  def test_get_normalized_http_url(self):
    url1 = 'HTTP://Example.com:80/resource?id=123'
    expected_url1 = "http://example.com/resource"
    self.oauth_request.http_url = url1
    self.assertEqual(self.oauth_request.get_normalized_http_url(), 
                     expected_url1)
    
    url2 = 'HTTPS://Example.com:443/resource?id=123'
    expected_url2 = "https://example.com/resource"
    self.oauth_request.http_url = url2
    self.assertEqual(self.oauth_request.get_normalized_http_url(), 
                     expected_url2)
    
    url3 = 'HTTP://Example.com:8080/resource?id=123'
    expected_url3 = "http://example.com:8080/resource"
    self.oauth_request.http_url = url3
    self.assertEqual(self.oauth_request.get_normalized_http_url(), 
                     expected_url3)
      
  def test_sign_request(self):
    expected_signature = self.oauth_request.parameters\
    .get(OAuthParameters.SIGNATURE)
    del self.oauth_request.parameters[OAuthParameters.SIGNATURE]
    self.oauth_request.sign_request(self.signature_method, 
                                    self.consumer, 
                                    self.token)
    self.assertEqual(self.oauth_request.parameters\
                     .get(OAuthParameters.SIGNATURE), expected_signature)

  def test_build_signature(self):
    expected_signature = self.oauth_request.parameters\
    .get(OAuthParameters.SIGNATURE)
    self.assertEqual(self.oauth_request.build_signature(self.signature_method, 
                                                        self.consumer,
                                                         self.token), 
                                                         expected_signature)
    
  def test_from_request(self):
    request = oauth.OAuthRequest.from_request(self.http_method, self.http_url, 
                                              self.oauth_header_with_realm, 
                                              {}, 
                                              "message=hey&context_id=")
    self.assertEqual(request.__dict__, self.oauth_request.__dict__)
    self.assertTrue(isinstance(request, oauth.OAuthRequest))
      
  def test_from_consumer_and_token(self):
    request = oauth.OAuthRequest.from_consumer_and_token(self.consumer, 
                                                         self.token, 
                                                         self.http_method, 
                                                         self.http_url)
    self.assertTrue(isinstance(request, oauth.OAuthRequest))
      
  def test_from_token_and_callback(self):
    callback = 'http://example.com'
    request = oauth.OAuthRequest.from_token_and_callback(self.token, 
                                                         callback, 
                                                         self.http_method, 
                                                         self.http_url)
    self.assertTrue(isinstance(request, oauth.OAuthRequest))
    self.assertEqual(request.get_parameter(OAuthParameters.CALLBACK), callback)

  def test__split_header(self):
    del self.parameters[self.non_oauth_param_message]
    del self.parameters[self.non_oauth_param_context_id]
    self.assertEqual(oauth.OAuthRequest._split_header(self\
        .oauth_header_with_realm['Authorization']), self.parameters)
    self.assertEqual(oauth.OAuthRequest._split_header(self\
        .oauth_header_without_realm['Authorization']), self.parameters)

  def test_split_url_string(self):
    qs = "a=1&c=hi%20there&empty="
    expected_result = {'a': '1',
                       'c': 'hi there',
                       'empty': ''}
    self.assertEqual(oauth.OAuthRequest._split_url_string(qs), expected_result)
        
class OAuthServerTest(OAuthTest):
  def setUp(self):
    super(OAuthServerTest, self).setUp()
    self.signature_method = oauth.OAuthSignatureMethod_HMAC_SHA1()
    self.data_store = MockOAuthDataStore()
    self.user = MockUser('Foo Bar')
    
    self.request_token_url = "http://example.com/oauth/request_token"
    self.access_token_url = "http://example.com/oauth/access_token"
    
    self.oauth_server = oauth.OAuthServer(self.data_store, 
                      {self.signature_method.get_name():self.signature_method})
      
  def _prepare_request(self, request, token = None):
    request.set_parameter(OAuthParameters.SIGNATURE_METHOD, 
                          self.signature_method.get_name())
    request.set_parameter(OAuthParameters.NONCE, oauth.generate_nonce())
    request.set_parameter(OAuthParameters.TIMESTAMP, 
                          oauth.generate_timestamp())
    request.sign_request(self.signature_method, self.consumer, token)
      
  def _get_token(self, request):
    self._prepare_request(request)
    return self.oauth_server.fetch_request_token(request)
  
  def _get_authorized_token(self, request):
    req_token = self._get_token(request)
    return self.oauth_server.authorize_token(req_token, self.user)
  
  def test_set_data_store(self):
    self.oauth_server.data_store = None
    self.assertTrue(self.oauth_server.data_store is None)
    self.oauth_server.set_data_store(self.data_store)
    self.assertTrue(self.oauth_server.data_store is not None)
    self.assertTrue(isinstance(self.oauth_server.data_store, 
                               oauth.OAuthDataStore))
      
  def test_get_data_store(self):
    self.assertEqual(self.oauth_server.data_store, self.data_store)
        
  def test_add_signature_method(self):
    signature_method = oauth.OAuthSignatureMethod_PLAINTEXT()
    self.oauth_server.add_signature_method(signature_method)
    self.assertTrue(isinstance(self.oauth_server.signature_methods\
                               .get(signature_method.get_name()), 
                               oauth.OAuthSignatureMethod_PLAINTEXT))
      
  def test_fetch_request_token(self):
    initial_request = oauth.OAuthRequest.from_consumer_and_token(
                                              self.consumer,
                                              http_method=self.http_method, 
                                              http_url=self.request_token_url
                                              )
    req_token_1 = self._get_token(initial_request)
    authorization_request = oauth.OAuthRequest.from_consumer_and_token(
                                                 self.consumer, 
                                                 req_token_1, 
                                                 http_method=self.http_method, 
                                                 http_url=self.http_url
                                                 )
    req_token_2 = self._get_token(authorization_request)
    self.assertEqual(req_token_1.key, req_token_2.key)
    self.assertEqual(req_token_1.secret, req_token_2.secret)
      
  def _get_token_for_authorization(self):
    request = oauth.OAuthRequest.from_consumer_and_token(
                                             self.consumer,
                                             http_method=self.http_method, 
                                             http_url=self.request_token_url
                                             )
    request_token = self._get_token(request)
    authorization_request = oauth.OAuthRequest.from_consumer_and_token(
                                                 self.consumer, 
                                                 request_token, 
                                                 http_method=self.http_method, 
                                                 http_url=self.http_url
                                                 )
    return self._get_authorized_token(authorization_request)
      
  def test_authorize_token(self):
    authorized_token = self._get_token_for_authorization()
    self.assertTrue(authorized_token is not None)
      
  def _get_access_token_request(self, authorized_token):
    access_token_request = oauth.OAuthRequest.from_consumer_and_token(
                                                self.consumer, 
                                                authorized_token, 
                                                http_method=self.http_method, 
                                                http_url=self.access_token_url
                                                )
    self._prepare_request(access_token_request, authorized_token)
    return access_token_request
      
  def test_fetch_access_token(self):
    authorized_token = self._get_token_for_authorization()
    access_token_request = self._get_access_token_request(authorized_token)
    access_token = self.oauth_server.fetch_access_token(access_token_request)
    self.assertTrue(access_token is not None)
    self.assertNotEqual(str(authorized_token), str(access_token))
    # Try to fetch access_token with used request token
    self.assertRaises(oauth.OAuthError, self.oauth_server.fetch_access_token, 
                      access_token_request)
      
  def test_verify_request(self):
    authorized_token = self._get_token_for_authorization()
    access_token_request = self._get_access_token_request(authorized_token)
    access_token = self.oauth_server.fetch_access_token(access_token_request)
    param1 = 'p1'
    value1 = 'v1'
    api_request = oauth.OAuthRequest.from_consumer_and_token(
                                                 self.consumer, 
                                                 access_token, 
                                                 http_method=self.http_method, 
                                                 http_url=self.http_url, 
                                                 parameters={param1:value1}
                                                 )
    self._prepare_request(api_request, access_token)
    result = self.oauth_server.verify_request(api_request)
    self.assertTrue(result is not None)
    consumer, token, parameters = result
    self.assertEqual(parameters.get(param1), value1)
  
  def test_get_callback(self):
    request = oauth.OAuthRequest.from_consumer_and_token(
                                                 self.consumer, 
                                                 None, 
                                                 http_method=self.http_method, 
                                                 http_url=self.http_url
                                                 )
    self._prepare_request(request)
    cb_url = 'http://example.com/cb'
    request.set_parameter(OAuthParameters.CALLBACK, cb_url)
    self.assertEqual(self.oauth_server.get_callback(request), cb_url)
      
  def test_build_authenticate_header(self):
    self.assertEqual(oauth.build_authenticate_header(), {'WWW-Authenticate': 
                                                         'OAuth realm=""'})
    self.assertEqual(oauth.build_authenticate_header('foo'), 
                     {'WWW-Authenticate': 'OAuth realm="foo"'})
        
class OAuthClientTest(OAuthTest):
  def setUp(self):
    super(OAuthClientTest, self).setUp()
    self.oauth_client = oauth.OAuthClient(self.consumer, self.token)
      
  def test_get_consumer(self):
    consumer = self.oauth_client.get_consumer()
    self.assertTrue(isinstance(consumer, oauth.OAuthConsumer))
    self.assertEqual(consumer.__dict__, self.consumer.__dict__)
      
  def test_get_token(self):
    token = self.oauth_client.get_token()
    self.assertTrue(isinstance(token, oauth.OAuthToken))
    self.assertEqual(token.__dict__, self.token.__dict__)
    
#Mockup OAuthDataStore
TOKEN_TYPE_REQUEST = 'request'
TOKEN_TYPE_ACCESS = 'access'
class MockOAuthDataStore(oauth.OAuthDataStore):
  def __init__(self):
    self.consumer = oauth.OAuthConsumer('a56b5ff0a637ab283d1d8e32ced37a9c', 
                                        '9a3248210c84b264b56b98c0b872bc8a')

    self.consumer_db = {self.consumer.key: self.consumer}
    self.request_token_db = {}
    self.access_token_db = {}
    self.nonce = None
      
  def lookup_consumer(self, key):
    return self.consumer_db.get(key)

  def lookup_token(self, oauth_consumer, token_type, token_field):
    data = None
    if token_type == TOKEN_TYPE_REQUEST:
      data = self.request_token_db.get(token_field)
    elif token_type == TOKEN_TYPE_ACCESS:
      data = self.access_token_db.get(token_field)
    
    if data:
        token, consumer, authenticated_user = data
        if consumer.key == oauth_consumer.key:
          return token
    return None

  def lookup_nonce(self, oauth_consumer, oauth_token, nonce):
    is_used = self.nonce == nonce
    self.nonce = nonce
    return is_used

  def fetch_request_token(self, oauth_consumer):
    token = oauth.OAuthToken("5b2cafbf20b11bace53b29e37d8a673dRT", 
                             "3f71254637df2002d8819458ae4f6c51RT")
    self.request_token_db[token.key] = (token, oauth_consumer, None)
    return token

  def fetch_access_token(self, oauth_consumer, oauth_token):
    data = self.request_token_db.get(oauth_token.key)
    if data:
      del self.request_token_db[oauth_token.key]
      request_token, consumer, authenticated_user = data
      access_token = oauth.OAuthToken("5b2cafbf20b11bace53b29e37d8a673dAT", 
                                      "3f71254637df2002d8819458ae4f6c51AT")
      self.access_token_db[access_token.key] = (access_token, 
                                                consumer, 
                                                authenticated_user)
      return access_token
    else:
      return None

  def authorize_request_token(self, oauth_token, user):
    data = self.request_token_db.get(oauth_token.key)
    if data and data[2] == None:
      request_token, consumer, authenticated_user = data
      authenticated_user = user
      self.request_token_db[request_token.key] = (request_token, 
                                                  consumer, 
                                                  authenticated_user)
      return request_token
    else:
      return None
    
#Mock user
class MockUser(object):
  def __init__(self, name):
    self.name = name
        
def suite():
  return test_config.build_suite([OauthUtilsTest, 
                                OAuthConsumerTest,
                                OAuthTokenTest,
                                OAuthRequestTest,
                                OAuthServerTest,
                                OAuthClientTest])

if __name__ == '__main__':
  unittest.main()


#!/usr/bin/python
#
# Copyright (C) 2008 Yu-Jie Lin
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import gdata.webmastertools.service
import gdata.service
try:
  from xml.etree import ElementTree
except ImportError:
  from elementtree import ElementTree
import atom
import getpass


username = ''
password = ''
site_uri = ''

username = raw_input('Please enter your username: ')
password = getpass.getpass()
site_uri = raw_input('Please enter your site url: ')

client = gdata.webmastertools.service.GWebmasterToolsService(
    email=username, 
    password=password, source='PythonWebmasterToolsSample-1')

print 'Logging in'
client.ProgrammaticLogin()

print 'Retrieving Sitemaps feed'
feed = client.GetSitemapsFeed(site_uri)

# Format the feed
print
print 'You have %d sitemap(s), last updated at %s' % (
    len(feed.entry), feed.updated.text)
print
print '='*80


def safeElementText(element):
  if hasattr(element, 'text'):
     return element.text
  return ''


# Format each site
for entry in feed.entry:
  print entry.title.text.replace('http://', '')[:80]
  print "  Last Updated   : %29s              Status: %10s" % (
      entry.updated.text[:29], entry.sitemap_status.text[:10])
  print "  Last Downloaded: %29s           URL Count: %10s" % (
      safeElementText(entry.sitemap_last_downloaded)[:29],
      safeElementText(entry.sitemap_url_count)[:10])
print



#!/usr/bin/python2.4
#
# Copyright 2011 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Live client tests for the Provisioning API."""

# This module is used for version 2 of the Google Data APIs.
# These tests attempt to connect to Google servers.


__author__ = 'Shraddha Gupta <shraddhag@google.com>'


import random
import unittest
import gdata.apps.client
import gdata.apps.data
import gdata.client
import gdata.data
import gdata.gauth
import gdata.test_config as conf

conf.options.register_option(conf.APPS_DOMAIN_OPTION)


class AppsClientTest(unittest.TestCase):

  def setUp(self):
    self.client = gdata.apps.client.AppsClient(
        domain='example.com')
    if conf.options.get_value('runlive') == 'true':
      self.client = gdata.apps.client.AppsClient(
          domain=conf.options.get_value('appsdomain'))
      if conf.options.get_value('ssl') == 'true':
        self.client.ssl = True
      conf.configure_client(self.client, 'AppsClientTest',
          self.client.auth_service, True)

  def tearDown(self):
    conf.close_client(self.client)

  def testClientConfiguration(self):
    self.assertEqual('apps-apis.google.com', self.client.host)
    self.assertEqual('2.0', self.client.api_version)
    self.assertEqual('apps', self.client.auth_service)
    self.assertEqual(
        ('https://apps-apis.google.com/a/feeds/user/',
         'https://apps-apis.google.com/a/feeds/policies/',
         'https://apps-apis.google.com/a/feeds/alias/',
         'https://apps-apis.google.com/a/feeds/groups/'),
        self.client.auth_scopes)
    if conf.options.get_value('runlive') == 'true':
      self.assertEqual(self.client.domain,
          conf.options.get_value('appsdomain'))
    else:
      self.assertEqual(self.client.domain, 'example.com')

  def testMakeUserProvisioningUri(self):
    self.assertEqual('/a/feeds/%s/user/2.0' % self.client.domain,
        self.client._userURL())

  def testMakeNicknameProvisioningUri(self):
    self.assertEqual('/a/feeds/%s/nickname/2.0' % self.client.domain,
        self.client._nicknameURL())

  def testCreateRetrieveUpdateDelete(self):
    if not conf.options.get_value('runlive') == 'true':
      return

    # Either load the recording or prepare to make a live request.
    conf.configure_cache(self.client, 'testCreateUpdateDelete')

    rnd_number = random.randrange(0, 100001)
    username = 'test_user%s' % (rnd_number)
    nickname = 'test_alias%s' % (rnd_number)

    new_entry = self.client.CreateUser(
        user_name=username, given_name='Elizabeth', family_name='Smith',
        password='password', admin='true')

    self.assert_(isinstance(new_entry,
        gdata.apps.data.UserEntry))
    self.assertEquals(new_entry.name.given_name, 'Elizabeth')
    self.assertEquals(new_entry.name.family_name, 'Smith')
    self.assertEquals(new_entry.login.user_name, username)
    self.assertEquals(new_entry.login.admin, 'true')

    fetched_entry = self.client.RetrieveUser(user_name=username)
    self.assertEquals(fetched_entry.name.given_name, 'Elizabeth')
    self.assertEquals(fetched_entry.name.family_name, 'Smith')
    self.assertEquals(fetched_entry.login.user_name, username)
    self.assertEquals(fetched_entry.login.admin, 'true')

    new_entry.name.given_name = 'Joe'
    new_entry.name.family_name = 'Brown'
    updated_entry = self.client.UpdateUser(
        user_name=username, user_entry=new_entry)
    self.assert_(isinstance(updated_entry,
        gdata.apps.data.UserEntry))
    self.assertEqual(updated_entry.name.given_name, 'Joe')
    self.assertEqual(updated_entry.name.family_name, 'Brown')

    new_nickname = self.client.CreateNickname(user_name=username,
        nickname=nickname)
    self.assert_(isinstance(new_nickname,
        gdata.apps.data.NicknameEntry))
    self.assertEquals(new_nickname.login.user_name, username)
    self.assertEquals(new_nickname.nickname.name, nickname)

    fetched_alias = self.client.RetrieveNickname(nickname)
    self.assertEquals(fetched_alias.login.user_name, username)
    self.assertEquals(fetched_alias.nickname.name, nickname)

    self.client.DeleteNickname(nickname)
    self.client.DeleteUser(username)


def suite():
  return conf.build_suite([AppsClientTest])


if __name__ == '__main__':
  unittest.TextTestRunner().run(suite())



#!/usr/bin/python
#
# Copyright (C) 2009 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


__author__ = 'api.rboyd@gmail.com (Ryan Boyd)'


try:
  from xml.etree import ElementTree
except ImportError:
  from elementtree import ElementTree
import gdata.calendar.data
import gdata.calendar.client
import gdata.acl.data
import atom
import getopt
import sys
import string
import time


class CalendarExample:

  def __init__(self, email, password):
    """Creates a CalendarService and provides ClientLogin auth details to it.
    The email and password are required arguments for ClientLogin.  The
    CalendarService automatically sets the service to be 'cl', as is
    appropriate for calendar.  The 'source' defined below is an arbitrary
    string, but should be used to reference your name or the name of your
    organization, the app name and version, with '-' between each of the three
    values.  The account_type is specified to authenticate either
    Google Accounts or Google Apps accounts.  See gdata.service or
    http://code.google.com/apis/accounts/AuthForInstalledApps.html for more
    info on ClientLogin.  NOTE: ClientLogin should only be used for installed
    applications and not for multi-user web applications."""

    self.cal_client = gdata.calendar.client.CalendarClient(source='Google-Calendar_Python_Sample-1.0')
    self.cal_client.ClientLogin(email, password, self.cal_client.source);

  def _PrintUserCalendars(self):
    """Retrieves the list of calendars to which the authenticated user either
    owns or subscribes to.  This is the same list as is represented in the
    Google Calendar GUI.  Although we are only printing the title of the
    calendar in this case, other information, including the color of the
    calendar, the timezone, and more.  See CalendarListEntry for more details
    on available attributes."""

    feed = self.cal_client.GetAllCalendarsFeed()
    print 'Printing allcalendars: %s' % feed.title.text
    for i, a_calendar in zip(xrange(len(feed.entry)), feed.entry):
      print '\t%s. %s' % (i, a_calendar.title.text,)

  def _PrintOwnCalendars(self):
    """Retrieves the list of calendars to which the authenticated user
    owns --
    Although we are only printing the title of the
    calendar in this case, other information, including the color of the
    calendar, the timezone, and more.  See CalendarListEntry for more details
    on available attributes."""

    feed = self.cal_client.GetOwnCalendarsFeed()
    print 'Printing owncalendars: %s' % feed.title.text
    for i, a_calendar in zip(xrange(len(feed.entry)), feed.entry):
      print '\t%s. %s' % (i, a_calendar.title.text,)

  def _PrintAllEventsOnDefaultCalendar(self):
    """Retrieves all events on the primary calendar for the authenticated user.
    In reality, the server limits the result set intially returned.  You can
    use the max_results query parameter to allow the server to send additional
    results back (see query parameter use in DateRangeQuery for more info).
    Additionally, you can page through the results returned by using the
    feed.GetNextLink().href value to get the location of the next set of
    results."""

    feed = self.cal_client.GetCalendarEventFeed()
    print 'Events on Primary Calendar: %s' % (feed.title.text,)
    for i, an_event in zip(xrange(len(feed.entry)), feed.entry):
      print '\t%s. %s' % (i, an_event.title.text,)
      for p, a_participant in zip(xrange(len(an_event.who)), an_event.who):
        print '\t\t%s. %s' % (p, a_participant.email,)
        print '\t\t\t%s' % (a_participant.value,)
        if a_participant.attendee_status:
          print '\t\t\t%s' % (a_participant.attendee_status.value,)

  def _FullTextQuery(self, text_query='Tennis'):
    """Retrieves events from the calendar which match the specified full-text
    query.  The full-text query searches the title and content of an event,
    but it does not search the value of extended properties at the time of
    this writing.  It uses the default (primary) calendar of the authenticated
    user and uses the private visibility/full projection feed.  Please see:
    http://code.google.com/apis/calendar/reference.html#Feeds
    for more information on the feed types.  Note: as we're not specifying
    any query parameters other than the full-text query, recurring events
    returned will not have gd:when elements in the response.  Please see
    the Google Calendar API query paramters reference for more info:
    http://code.google.com/apis/calendar/reference.html#Parameters"""

    print 'Full text query for events on Primary Calendar: \'%s\'' % (
        text_query,)
    query = gdata.calendar.client.CalendarEventQuery(text_query=text_query)
    feed = self.cal_client.GetCalendarEventFeed(q=query)
    for i, an_event in zip(xrange(len(feed.entry)), feed.entry):
      print '\t%s. %s' % (i, an_event.title.text,)
      print '\t\t%s. %s' % (i, an_event.content.text,)
      for a_when in an_event.when:
        print '\t\tStart time: %s' % (a_when.start,)
        print '\t\tEnd time:   %s' % (a_when.end,)

  def _DateRangeQuery(self, start_date='2007-01-01', end_date='2007-07-01'):
    """Retrieves events from the server which occur during the specified date
    range.  This uses the CalendarEventQuery class to generate the URL which is
    used to retrieve the feed.  For more information on valid query parameters,
    see: http://code.google.com/apis/calendar/reference.html#Parameters"""

    print 'Date range query for events on Primary Calendar: %s to %s' % (
        start_date, end_date,)
    query = gdata.calendar.client.CalendarEventQuery(start_min=start_date, start_max=end_date)
    feed = self.cal_client.GetCalendarEventFeed(q=query)
    for i, an_event in zip(xrange(len(feed.entry)), feed.entry):
      print '\t%s. %s' % (i, an_event.title.text,)
      for a_when in an_event.when:
        print '\t\tStart time: %s' % (a_when.start,)
        print '\t\tEnd time:   %s' % (a_when.end,)

  def _InsertCalendar(self, title='Little League Schedule',
      description='This calendar contains practice and game times',
      time_zone='America/Los_Angeles', hidden=False, location='Oakland',
      color='#2952A3'):
    """Creates a new calendar using the specified data."""
    print 'Creating new calendar with title "%s"' % title
    calendar = gdata.calendar.data.CalendarEntry()
    calendar.title = atom.data.Title(text=title)
    calendar.summary = atom.data.Summary(text=description)
    calendar.where.append(gdata.calendar.data.CalendarWhere(value=location))
    calendar.color = gdata.calendar.data.ColorProperty(value=color)
    calendar.timezone = gdata.calendar.data.TimeZoneProperty(value=time_zone)

    if hidden:
      calendar.hidden = gdata.calendar.data.HiddenProperty(value='true')
    else:
      calendar.hidden = gdata.calendar.data.HiddenProperty(value='false')

    new_calendar = self.cal_client.InsertCalendar(new_calendar=calendar)
    return new_calendar

  def _UpdateCalendar(self, calendar, title='New Title', color=None):
    """Updates the title and, optionally, the color of the supplied calendar"""
    print 'Updating the calendar titled "%s" with the title "%s"' % (
        calendar.title.text, title)
    calendar.title = atom.data.Title(text=title)
    if color is not None:
      calendar.color = gdata.calendar.data.ColorProperty(value=color)

    updated_calendar = self.cal_client.Update(calendar)
    return updated_calendar

  def _DeleteAllCalendars(self):
    """Deletes all calendars.  Note: the primary calendar cannot be deleted"""
    feed = self.cal_client.GetOwnCalendarsFeed()
    for entry in feed.entry:
      print 'Deleting calendar: %s' % entry.title.text
      try:
        self.cal_client.Delete(entry.GetEditLink().href)
      except gdata.client.RequestError, msg:
        if msg.body.startswith('Cannot remove primary calendar'):
            print '\t%s' % msg.body
        else:
            print '\tUnexpected Error: %s' % msg.body

  def _InsertSubscription(self,
      id='python.gcal.test%40gmail.com'):
    """Subscribes to the calendar with the specified ID."""
    print 'Subscribing to the calendar with ID: %s' % id
    calendar = gdata.calendar.data.CalendarEntry()
    calendar.id = atom.data.Id(text=id)
    returned_calendar = self.cal_client.InsertCalendarSubscription(calendar)
    return returned_calendar

  def _UpdateCalendarSubscription(self,
      id='python.gcal.test%40gmail.com',
      color=None, hidden=None, selected=None):
    """Updates the subscription to the calendar with the specified ID."""
    print 'Updating the calendar subscription with ID: %s' % id
    calendar_url = (
      'http://www.google.com/calendar/feeds/default/allcalendars/full/%s' % id)
    calendar_entry = self.cal_client.GetCalendarEntry(calendar_url)

    if color is not None:
      calendar_entry.color = gdata.calendar.data.ColorProperty(value=color)
    if hidden is not None:
      if hidden:
        calendar_entry.hidden = gdata.calendar.data.HiddenProperty(value='true')
      else:
        calendar_entry.hidden = gdata.calendar.data.HiddenProperty(value='false')
    if selected is not None:
      if selected:
        calendar_entry.selected = gdata.calendar.data.SelectedProperty(value='true')
      else:
        calendar_entry.selected = gdata.calendar.data.SelectedProperty(value='false')

    updated_calendar = self.cal_client.Update(calendar_entry)
    return updated_calendar

  def _DeleteCalendarSubscription(self,
      id='python.gcal.test%40gmail.com'):
    """Deletes the subscription to the calendar with the specified ID."""
    print 'Deleting the calendar subscription with ID: %s' % id
    calendar_url = (
      'http://www.google.com/calendar/feeds/default/allcalendars/full/%s' % id)
    calendar_entry = self.cal_client.GetCalendarEntry(calendar_url)
    self.cal_client.Delete(calendar_entry.GetEditLink().href)

  def _InsertEvent(self, title='Tennis with Beth',
      content='Meet for a quick lesson', where='On the courts',
      start_time=None, end_time=None, recurrence_data=None):
    """Inserts a basic event using either start_time/end_time definitions
    or gd:recurrence RFC2445 icalendar syntax.  Specifying both types of
    dates is not valid.  Note how some members of the CalendarEventEntry
    class use arrays and others do not.  Members which are allowed to occur
    more than once in the calendar or GData "kinds" specifications are stored
    as arrays.  Even for these elements, Google Calendar may limit the number
    stored to 1.  The general motto to use when working with the Calendar data
    API is that functionality not available through the GUI will not be
    available through the API.  Please see the GData Event "kind" document:
    http://code.google.com/apis/gdata/elements.html#gdEventKind
    for more information"""

    event = gdata.calendar.data.CalendarEventEntry()
    event.title = atom.data.Title(text=title)
    event.content = atom.data.Content(text=content)
    event.where.append(gdata.data.Where(value=where))

    if recurrence_data is not None:
      # Set a recurring event
      event.recurrence = gdata.data.Recurrence(text=recurrence_data)
    else:
      if start_time is None:
        # Use current time for the start_time and have the event last 1 hour
        start_time = time.strftime('%Y-%m-%dT%H:%M:%S.000Z', time.gmtime())
        end_time = time.strftime('%Y-%m-%dT%H:%M:%S.000Z',
            time.gmtime(time.time() + 3600))
      event.when.append(gdata.data.When(start=start_time,
          end=end_time))

    new_event = self.cal_client.InsertEvent(event)

    return new_event

  def _InsertSingleEvent(self, title='One-time Tennis with Beth',
      content='Meet for a quick lesson', where='On the courts',
      start_time=None, end_time=None):
    """Uses the _InsertEvent helper method to insert a single event which
    does not have any recurrence syntax specified."""

    new_event = self._InsertEvent(title, content, where, start_time, end_time,
        recurrence_data=None)

    print 'New single event inserted: %s' % (new_event.id.text,)
    print '\tEvent edit URL: %s' % (new_event.GetEditLink().href,)
    print '\tEvent HTML URL: %s' % (new_event.GetHtmlLink().href,)

    return new_event

  def _InsertRecurringEvent(self, title='Weekly Tennis with Beth',
      content='Meet for a quick lesson', where='On the courts',
      recurrence_data=None):
    """Uses the _InsertEvent helper method to insert a recurring event which
    has only RFC2445 icalendar recurrence syntax specified.  Note the use of
    carriage return/newline pairs at the end of each line in the syntax.  Even
    when specifying times (as opposed to only dates), VTIMEZONE syntax is not
    required if you use a standard Java timezone ID.  Please see the docs for
    more information on gd:recurrence syntax:
    http://code.google.com/apis/gdata/elements.html#gdRecurrence
    """

    if recurrence_data is None:
      recurrence_data = ('DTSTART;VALUE=DATE:20070501\r\n'
        + 'DTEND;VALUE=DATE:20070502\r\n'
        + 'RRULE:FREQ=WEEKLY;BYDAY=Tu;UNTIL=20070904\r\n')

    new_event = self._InsertEvent(title, content, where,
        recurrence_data=recurrence_data, start_time=None, end_time=None)

    print 'New recurring event inserted: %s' % (new_event.id.text,)
    print '\tEvent edit URL: %s' % (new_event.GetEditLink().href,)
    print '\tEvent HTML URL: %s' % (new_event.GetHtmlLink().href,)

    return new_event

  def _InsertQuickAddEvent(self,
      content="Tennis with John today 3pm-3:30pm"):
    """Creates an event with the quick_add property set to true so the content
    is processed as quick add content instead of as an event description."""
    event = gdata.calendar.data.CalendarEventEntry()
    event.content = atom.data.Content(text=content)
    event.quick_add = gdata.calendar.data.QuickAddProperty(value='true')

    new_event = self.cal_client.InsertEvent(event)
    return new_event

  def _InsertSimpleWebContentEvent(self):
    """Creates a WebContent object and embeds it in a WebContentLink.
    The WebContentLink is appended to the existing list of links in the event
    entry.  Finally, the calendar client inserts the event."""

    # Create a WebContent object
    url = 'http://www.google.com/logos/worldcup06.gif'
    web_content = gdata.calendar.data.WebContent(url=url, width='276', height='120')

    # Create a WebContentLink object that contains the WebContent object
    title = 'World Cup'
    href = 'http://www.google.com/calendar/images/google-holiday.gif'
    type = 'image/gif'
    web_content_link = gdata.calendar.data.WebContentLink(title=title, href=href,
        link_type=type, web_content=web_content)

    # Create an event that contains this web content
    event = gdata.calendar.data.CalendarEventEntry()
    event.link.append(web_content_link)

    print 'Inserting Simple Web Content Event'
    new_event = self.cal_client.InsertEvent(event)
    return new_event

  def _InsertWebContentGadgetEvent(self):
    """Creates a WebContent object and embeds it in a WebContentLink.
    The WebContentLink is appended to the existing list of links in the event
    entry.  Finally, the calendar client inserts the event.  Web content
    gadget events display Calendar Gadgets inside Google Calendar."""

    # Create a WebContent object
    url = 'http://google.com/ig/modules/datetime.xml'
    web_content = gdata.calendar.data.WebContent(url=url, width='300', height='136')
    web_content.web_content_gadget_pref.append(
        gdata.calendar.data.WebContentGadgetPref(name='color', value='green'))

    # Create a WebContentLink object that contains the WebContent object
    title = 'Date and Time Gadget'
    href = 'http://gdata.ops.demo.googlepages.com/birthdayicon.gif'
    type = 'application/x-google-gadgets+xml'
    web_content_link = gdata.calendar.data.WebContentLink(title=title, href=href,
        link_type=type, web_content=web_content)

    # Create an event that contains this web content
    event = gdata.calendar.data.CalendarEventEntry()
    event.link.append(web_content_link)

    print 'Inserting Web Content Gadget Event'
    new_event = self.cal_client.InsertEvent(event)
    return new_event

  def _UpdateTitle(self, event, new_title='Updated event title'):
    """Updates the title of the specified event with the specified new_title.
    Note that the UpdateEvent method (like InsertEvent) returns the
    CalendarEventEntry object based upon the data returned from the server
    after the event is inserted.  This represents the 'official' state of
    the event on the server.  The 'edit' link returned in this event can
    be used for future updates.  Due to the use of the 'optimistic concurrency'
    method of version control, most GData services do not allow you to send
    multiple update requests using the same edit URL.  Please see the docs:
    http://code.google.com/apis/gdata/reference.html#Optimistic-concurrency
    """

    previous_title = event.title.text
    event.title.text = new_title
    print 'Updating title of event from:\'%s\' to:\'%s\'' % (
        previous_title, event.title.text,)
    return self.cal_client.Update(event)

  def _AddReminder(self, event, minutes=10):
    """Adds a reminder to the event.  This uses the default reminder settings
    for the user to determine what type of notifications are sent (email, sms,
    popup, etc.) and sets the reminder for 'minutes' number of minutes before
    the event.  Note: you can only use values for minutes as specified in the
    Calendar GUI."""

    for a_when in event.when:
      if len(a_when.reminder) > 0:
        a_when.reminder[0].minutes = minutes
      else:
        a_when.reminder.append(gdata.data.Reminder(minutes=str(minutes)))

    print 'Adding %d minute reminder to event' % (minutes,)
    return self.cal_client.Update(event)

  def _AddExtendedProperty(self, event,
      name='http://www.example.com/schemas/2005#mycal.id', value='1234'):
    """Adds an arbitrary name/value pair to the event.  This value is only
    exposed through the API.  Extended properties can be used to store extra
    information needed by your application.  The recommended format is used as
    the default arguments above.  The use of the URL format is to specify a
    namespace prefix to avoid collisions between different applications."""

    event.extended_property.append(
        gdata.calendar.data.CalendarExtendedProperty(name=name, value=value))
    print 'Adding extended property to event: \'%s\'=\'%s\'' % (name, value,)
    return self.cal_client.Update(event)

  def _DeleteEvent(self, event):
    """Given an event object returned for the calendar server, this method
    deletes the event.  The edit link present in the event is the URL used
    in the HTTP DELETE request."""

    self.cal_client.Delete(event.GetEditLink().href)

  def _PrintAclFeed(self):
    """Sends a HTTP GET to the default ACL URL
    (http://www.google.com/calendar/feeds/default/acl/full) and displays the
    feed returned in the response."""

    feed = self.cal_client.GetCalendarAclFeed()
    print feed.title.text
    for i, a_rule in zip(xrange(len(feed.entry)), feed.entry):
      print '\t%s. %s' % (i, a_rule.title.text,)
      print '\t\t Role: %s' % (a_rule.role.value,)
      print '\t\t Scope %s - %s' % (a_rule.scope.type, a_rule.scope.value)

  def _CreateAclRule(self, username):
    """Creates a ACL rule that grants the given user permission to view
    free/busy information on the default calendar.  Note: It is not necessary
    to specify a title for the ACL entry.  The server will set this to be the
    value of the role specified (in this case "freebusy")."""

    print 'Creating Acl rule for user: %s' % username
    rule = gdata.calendar.data.CalendarAclEntry()
    rule.scope = gdata.acl.data.AclScope(value=username, type="user")
    roleValue = "http://schemas.google.com/gCal/2005#%s" % ("freebusy")
    rule.role = gdata.acl.data.AclRole(value=roleValue)
    aclUrl = "https://www.google.com/calendar/feeds/default/acl/full"
    returned_rule = self.cal_client.InsertAclEntry(rule, aclUrl)

  def _RetrieveAclRule(self, username):
    """Builds the aclEntryUri or the entry created in the previous example.
    The sends a HTTP GET message and displays the entry returned in the
    response."""

    aclEntryUri = "http://www.google.com/calendar/feeds/"
    aclEntryUri += "default/acl/full/user:%s" % (username)
    entry = self.cal_client.GetCalendarAclEntry(aclEntryUri)
    print '\t%s' % (entry.title.text,)
    print '\t\t Role: %s' % (entry.role.value,)
    print '\t\t Scope %s - %s' % (entry.scope.type, entry.scope.value)
    return entry

  def _UpdateAclRule(self, entry):
    """Modifies the value of the role in the given entry and POSTs the updated
    entry.  Note that while the role of an ACL entry can be updated, the
    scope can not be modified."""

    print 'Update Acl rule: %s' % (entry.GetEditLink().href)
    roleValue = "http://schemas.google.com/gCal/2005#%s" % ("read")
    entry.role = gdata.acl.data.AclRole(value=roleValue)
    returned_rule = self.cal_client.Update(entry)

  def _DeleteAclRule(self, entry):
    """Given an ACL entry returned for the calendar server, this method
    deletes the entry.  The edit link present in the entry is the URL used
    in the HTTP DELETE request."""

    self.cal_client.Delete(entry.GetEditLink().href)

  def _batchRequest(self, updateEntry, deleteEntry):
    """Execute a batch request to create, update and delete an entry."""

    print 'Executing batch request to insert, update and delete entries.'
    # feed that holds all the batch rquest entries
    request_feed = gdata.calendar.data.CalendarEventFeed()

    # creating an event entry to insert
    insertEntry = gdata.calendar.data.CalendarEventEntry()
    insertEntry.title = atom.data.Title(text='Python: batch insert')
    insertEntry.content = atom.data.Content(text='my content')
    start_time = time.strftime('%Y-%m-%dT%H:%M:%S.000Z', time.gmtime())
    end_time = time.strftime('%Y-%m-%dT%H:%M:%S.000Z',
                             time.gmtime(time.time() + 3600))
    insertEntry.when.append(gdata.calendar.data.When(start=start_time,
                                                     end=end_time))
    insertEntry.batch_id = gdata.data.BatchId(text='insert-request')

    # add the insert entry to the batch feed
    request_feed.AddInsert(entry=insertEntry)

    if updateEntry:
      updateEntry.batch_id = gdata.data.BatchId(text='update-request')
      updateEntry.title = atom.data.Title(text='Python: batch update')
      # add the update entry to the batch feed
      request_feed.AddUpdate(entry=updateEntry)

    if deleteEntry:
      deleteEntry.batch_id = gdata.data.BatchId(text='delete-request')
      # add the delete entry to the batch feed
      request_feed.AddDelete(entry=deleteEntry)

    # submit the batch request to the server
    response_feed = self.cal_client.ExecuteBatch(request_feed, gdata.calendar.client.DEFAULT_BATCH_URL)

    # iterate the response feed to get the operation status
    for entry in response_feed.entry:
      print '\tbatch id: %s' % (entry.batch_id.text,)
      print '\tstatus: %s' % (entry.batch_status.code,)
      print '\treason: %s' % (entry.batch_status.reason,)
      if entry.batch_id.text == 'insert-request':
        insertEntry = entry
      elif entry.batch_id.text == 'update-request':
        updateEntry = entry

    return (insertEntry, updateEntry)

  def Run(self, delete='false'):
    """Runs each of the example methods defined above.  Note how the result
    of the _InsertSingleEvent call is used for updating the title and the
    result of updating the title is used for inserting the reminder and
    again with the insertion of the extended property.  This is due to the
    Calendar's use of GData's optimistic concurrency versioning control system:
    http://code.google.com/apis/gdata/reference.html#Optimistic-concurrency
    """

    # Getting feeds and query results
    self._PrintUserCalendars()
    self._PrintOwnCalendars()
    self._PrintAllEventsOnDefaultCalendar()
    self._FullTextQuery()
    self._DateRangeQuery()

    # Inserting and updating events
    see = self._InsertSingleEvent()
    see_u_title = self._UpdateTitle(see, 'New title for single event')
    see_u_reminder = self._AddReminder(see_u_title, minutes=30)
    see_u_ext_prop = self._AddExtendedProperty(see_u_reminder,
        name='propname', value='propvalue')
    ree = self._InsertRecurringEvent()
    simple_web_content_event = self._InsertSimpleWebContentEvent()
    web_content_gadget_event = self._InsertWebContentGadgetEvent()
    quick_add_event = self._InsertQuickAddEvent()

    # Access Control List examples
    self._PrintAclFeed()
    self._CreateAclRule("user@gmail.com")
    entry = self._RetrieveAclRule("user@gmail.com")
    self._UpdateAclRule(entry)
    self._DeleteAclRule(entry)

    # Creating, updating and deleting calendars
    inserted_calendar = self._InsertCalendar()
    updated_calendar = self._UpdateCalendar(calendar=inserted_calendar)

    # Insert Subscription
    inserted_subscription = self._InsertSubscription()
    updated_subscription = self._UpdateCalendarSubscription(selected=False)

    # Execute a batch request
    (quick_add_event, see_u_ext_prop) = self._batchRequest(see_u_ext_prop,
                                                           quick_add_event)

    # Delete entries if delete argument='true'
    if delete == 'true':
      print 'Deleting created events'
      self.cal_client.Delete(see_u_ext_prop)
      self.cal_client.Delete(ree)
      self.cal_client.Delete(simple_web_content_event)
      self.cal_client.Delete(web_content_gadget_event)
      self.cal_client.Delete(quick_add_event)
      print 'Deleting subscriptions'
      self._DeleteCalendarSubscription()
      print 'Deleting all calendars'
      self._DeleteAllCalendars()


def main():
  """Runs the CalendarExample application with the provided username and
  and password values.  Authentication credentials are required.
  NOTE: It is recommended that you run this sample using a test account."""

  # parse command line options
  try:
    opts, args = getopt.getopt(sys.argv[1:], "", ["user=", "pw=", "delete="])
  except getopt.error, msg:
    print ('python calendarExample.py --user [username] --pw [password] ' +
        '--delete [true|false] ')
    sys.exit(2)

  user = ''
  pw = ''
  delete = 'false'

  # Process options
  for o, a in opts:
    if o == "--user":
      user = a
    elif o == "--pw":
      pw = a
    elif o == "--delete":
      delete = a

  if user == '' or pw == '':
    print ('python calendarExample.py --user [username] --pw [password] ' +
        '--delete [true|false] ')
    sys.exit(2)

  sample = CalendarExample(user, pw)
  sample.Run(delete)

if __name__ == '__main__':
  main()



#!/usr/bin/python
#
# Copyright (C) 2007 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


__author__ = 'api.jscudder (Jeffrey Scudder)'


import unittest
import gdata.codesearch
import gdata.test_data

class CodeSearchDataTest(unittest.TestCase):

  def setUp(self):
   self.feed = gdata.codesearch.CodesearchFeedFromString(
       gdata.test_data.CODE_SEARCH_FEED)

  def testCorrectXmlConversion(self):
    self.assert_(self.feed.id.text == 
        'http://www.google.com/codesearch/feeds/search?q=malloc')
    self.assert_(len(self.feed.entry) == 10)
    for entry in self.feed.entry:
      if entry.id.text == ('http://www.google.com/codesearch?hl=en&q=+ma'
          'lloc+show:LDjwp-Iqc7U:84hEYaYsZk8:xDGReDhvNi0&sa=N&ct=rx&cd=1'
          '&cs_p=http://www.gnu.org&cs_f=software/autoconf/manual/autoco'
          'nf-2.60/autoconf.html-002&cs_p=http://www.gnu.org&cs_f=softwa'
          're/autoconf/manual/autoconf-2.60/autoconf.html-002#first'):
        self.assert_(len(entry.match) == 4)
        for match in entry.match:
          if match.line_number == '4':
            self.assert_(match.type == 'text/html')
        self.assert_(entry.file.name == 
            'software/autoconf/manual/autoconf-2.60/autoconf.html-002')
        self.assert_(entry.package.name == 'http://www.gnu.org')
        self.assert_(entry.package.uri == 'http://www.gnu.org')

    
if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
#
# Copyright (C) 2009 Tan Swee Heng
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


__author__ = 'thesweeheng@gmail.com'


from gdata.finance.service import \
    FinanceService, PortfolioQuery, PositionQuery
from gdata.finance import \
    PortfolioEntry, PortfolioData, TransactionEntry, TransactionData, \
    Price, Commission, Money
import datetime
import sys


def PrintReturns(pfx, d):
  """Print returns."""
  print pfx, '%1.5f(1w)  %1.5f(4w)  %1.5f(3m)  %1.5f(YTD)' % tuple(
    float(i) for i in (d.return1w, d.return4w, d.return3m, d.returnYTD))
  pfx = ' ' * len(pfx)
  print pfx, '%1.5f(1y)  %1.5f(3y)  %1.5f(5y)  %1.5f(overall)' % tuple(
    float(i) for i in (d.return1y, d.return3y, d.return5y, d.return_overall))


PrRtn = PrintReturns


def PrintTransactions(transactions):
  """Print transactions."""
  print "    Transactions:"
  fmt = '    %4s %-23s %-10s %6s %-11s %-11s'
  print fmt % ('ID','Date','Type','Shares','Price','Commission')
  for txn in transactions:
    d = txn.transaction_data
    print fmt % (txn.transaction_id, d.date or '----', d.type,
        d.shares, d.price.money[0], d.commission.money[0])
    if d.notes:
        print "         Notes:", d.notes
  print


def PrintPosition(pos, with_returns=False):
  """Print single position."""
  print '    Position      :', pos.position_title
  print '    Ticker ID     :', pos.ticker_id
  print '    Symbol        :', pos.symbol
  print '    Last updated  :', pos.updated.text
  d = pos.position_data
  print '    Shares        :', d.shares
  if with_returns:
    print '    Gain %        :', d.gain_percentage
    PrRtn('    Returns       :', d)
    print '    Cost basis    :', d.cost_basis
    print '    Days gain     :', d.days_gain
    print '    Gain          :', d.gain
    print '    Market value  :', d.market_value
  print
  if pos.transactions:
    print "    <inlined transactions>\n"
    PrintTransactions(pos.transactions)
    print "    </inlined transactions>\n"


def PrintPositions(positions, with_returns=False):
  for pos in positions:
    PrintPosition(pos, with_returns)


def PrintPortfolio(pfl, with_returns=False):
  """Print single portfolio."""
  print 'Portfolio Title   :', pfl.portfolio_title
  print 'Portfolio ID      :', pfl.portfolio_id
  print '  Last updated    :', pfl.updated.text
  d = pfl.portfolio_data
  print '  Currency        :', d.currency_code
  if with_returns:
    print '  Gain %          :', d.gain_percentage
    PrRtn('  Returns         :', d)
    print '  Cost basis      :', d.cost_basis
    print '  Days gain       :', d.days_gain
    print '  Gain            :', d.gain
    print '  Market value    :', d.market_value
  print
  if pfl.positions:
    print "  <inlined positions>\n"
    PrintPositions(pfl.positions, with_returns)
    print "  </inlined positions>\n"


def PrintPortfolios(portfolios, with_returns=False):
  for pfl in portfolios:
    PrintPortfolio(pfl, with_returns)


def ShowCallDetails(meth):
  def wrap(*args, **kwargs):
    print '@', meth.__name__, args[1:], kwargs
    meth(*args, **kwargs)
  return wrap


class FinanceTester(object):

  def __init__(self, email, password):
    self.client = FinanceService(source='gdata-finance-test')
    self.client.ClientLogin(email, password)
  
  def GetPortfolios(self, with_returns=False, inline_positions=False): 
    query = PortfolioQuery()
    query.returns = with_returns
    query.positions = inline_positions
    return self.client.GetPortfolioFeed(query=query).entry

  def GetPositions(self, portfolio, with_returns=False, inline_transactions=False):
    query = PositionQuery()
    query.returns = with_returns
    query.transactions = inline_transactions
    return self.client.GetPositionFeed(portfolio, query=query).entry

  def GetTransactions(self, position=None, portfolio=None, ticker=None):
    if position:
      feed = self.client.GetTransactionFeed(position)
    elif portfolio and ticker:
      feed = self.client.GetTransactionFeed(
          portfolio_id=portfolio.portfolio_id, ticker_id=ticker)
    return feed.entry

  @ShowCallDetails
  def TestShowDetails(self, with_returns=False, inline_positions=False,
      inline_transactions=False):
    portfolios = self.GetPortfolios(with_returns, inline_positions)
    for pfl in portfolios:
      PrintPortfolio(pfl, with_returns)
      positions = self.GetPositions(pfl, with_returns, inline_transactions)
      for pos in positions:
        PrintPosition(pos, with_returns)
        PrintTransactions(self.GetTransactions(pos))

  def DeletePortfoliosByName(self, portfolio_titles):
    for pfl in self.GetPortfolios():
      if pfl.portfolio_title in portfolio_titles:
        self.client.DeletePortfolio(pfl)

  def AddPortfolio(self, portfolio_title, currency_code):
    pfl = PortfolioEntry(portfolio_data=PortfolioData(
        currency_code=currency_code))
    pfl.portfolio_title = portfolio_title
    return self.client.AddPortfolio(pfl)

  def UpdatePortfolio(self, portfolio,
      portfolio_title=None, currency_code=None):
    if portfolio_title:
      portfolio.portfolio_title = portfolio_title
    if currency_code:
      portfolio.portfolio_data.currency_code = currency_code
    return self.client.UpdatePortfolio(portfolio)

  def DeletePortfolio(self, portfolio):
    self.client.DeletePortfolio(portfolio)

  @ShowCallDetails
  def TestManagePortfolios(self):
    pfl_one = 'Portfolio Test: Emerging Markets 12345'
    pfl_two = 'Portfolio Test: Renewable Energy 31415'

    print '---- Deleting portfolios ----'
    self.DeletePortfoliosByName([pfl_one, pfl_two])
    PrintPortfolios(self.GetPortfolios())
    print '---- Adding new portfolio ----'
    pfl = self.AddPortfolio(pfl_one, 'SGD')
    PrintPortfolios(self.GetPortfolios())
    print '---- Changing portfolio title and currency code ----'
    pfl = self.UpdatePortfolio(pfl, pfl_two, 'USD')
    PrintPortfolios(self.GetPortfolios())
    print '---- Deleting portfolio ----'
    self.DeletePortfolio(pfl)
    PrintPortfolios(self.GetPortfolios())

  def Transact(self, type, portfolio, ticker, date=None, shares=None,
      notes=None, price=None, commission=None, currency_code=None):
    if price is not None:
      price = Price(money=[Money(amount=str(price),
          currency_code=currency_code or
          portfolio.portfolio_data.currency_code)])
    if commission is not None:
      commission = Commission(money=[Money(amount=str(comission),
          currency_code=currency_code or
          portfolio.portfolio_data.currency_code)])
    if date is not None and isinstance(date, datetime.datetime):
      date = date.isoformat()
    if shares is not None:
      shares = str(shares)
    txn = TransactionEntry(transaction_data=TransactionData(type=type,
        date=date, shares=shares, notes=notes, price=price,
        commission=commission))
    return self.client.AddTransaction(txn,
        portfolio_id=portfolio.portfolio_id, ticker_id=ticker)
  
  def Buy(self, portfolio, ticker, **kwargs):
    return self.Transact('Buy', portfolio, ticker, **kwargs)
  
  def Sell(self, portfolio, ticker, **kwargs):
    return self.Transact('Sell', portfolio, ticker, **kwargs)

  def GetPosition(self, portfolio, ticker, with_returns=False, inline_transactions=False):
    query = PositionQuery()
    query.returns = with_returns
    query.transactions = inline_transactions
    return self.client.GetPosition(
        portfolio_id=portfolio.portfolio_id, ticker_id=ticker, query=query)

  def DeletePosition(self, position):
    self.client.DeletePosition(position_entry=position)

  def UpdateTransaction(self, transaction):
    self.client.UpdateTransaction(transaction)

  def DeleteTransaction(self, transaction):
    self.client.DeleteTransaction(transaction)

  @ShowCallDetails
  def TestManageTransactions(self):
    pfl_title = 'Transaction Test: Technology 27182'
    self.DeletePortfoliosByName([pfl_title])

    print '---- Adding new portfolio ----'
    pfl = self.AddPortfolio(pfl_title, 'USD')
    PrintPortfolios(self.GetPortfolios())

    print '---- Adding buy transactions ----'
    tkr1 = 'NASDAQ:GOOG'
    date = datetime.datetime(2009,04,01)
    days = datetime.timedelta(1)
    txn1 = self.Buy(pfl, tkr1, shares=500, price=321.00, date=date)
    txn2 = self.Buy(pfl, tkr1, shares=150, price=312.00, date=date+15*days)
    pos = self.GetPosition(portfolio=pfl, ticker=tkr1, with_returns=True)
    PrintPosition(pos, with_returns=True)
    PrintTransactions(self.GetTransactions(pos))

    print '---- Adding sell transactions ----'
    txn3 = self.Sell(pfl, tkr1, shares=400, price=322.00, date=date+30*days)
    txn4 = self.Sell(pfl, tkr1, shares=200, price=330.00, date=date+45*days)
    pos = self.GetPosition(portfolio=pfl, ticker=tkr1, with_returns=True)
    PrintPosition(pos, with_returns=True)
    PrintTransactions(self.GetTransactions(pos))

    print "---- Modifying first and deleting third ----"
    txn1.transaction_data.shares = '400.0'
    self.UpdateTransaction(txn1)
    self.DeleteTransaction(txn3)
    pos = self.GetPosition(portfolio=pfl, ticker=tkr1, with_returns=True)
    PrintPosition(pos, with_returns=True)
    PrintTransactions(self.GetTransactions(pos))

    print "---- Deleting position ----"
    print "Number of positions (before):", len(self.GetPositions(pfl))
    self.DeletePosition(pos)
    print "Number of positions (after) :", len(self.GetPositions(pfl))

    print '---- Deleting portfolio ----'
    self.DeletePortfolio(pfl)
    PrintPortfolios(self.GetPortfolios())


if __name__ == '__main__':
  try:
    email = sys.argv[1]
    password = sys.argv[2]
    cases = sys.argv[3:]
  except IndexError:
    print "Usage: test_finance account@google.com password [0 1 2...]"
    sys.exit(1)

  tester = FinanceTester(email, password)
  tests = [
      tester.TestShowDetails,
      lambda: tester.TestShowDetails(with_returns=True),
      tester.TestManagePortfolios,
      tester.TestManageTransactions,
      lambda: tester.TestShowDetails(with_returns=True, inline_positions=True),
      lambda: tester.TestShowDetails(with_returns=True, inline_positions=True,
          inline_transactions=True),]
  if not cases:
    cases = range(len(tests))
  for i in cases:
    print "===== TEST CASE", i, "="*50
    tests[int(i)]()
